"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _utils = require("../../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
  // gets value or values of a given object, from a name or namespace (example: "dog.name")
  getPropertyValue: function getPropertyValue(o, name) {
    var a = name.split("."),
        x = o,
        p;
    while (p = a.shift()) {
      if (_utils2.default.has(x, p)) {
        x = x[p];
      }
      if (_utils2.default.isArray(x)) {
        break;
      }
    }
    if (_utils2.default.isArray(x)) {
      if (!a.length) {
        return x;
      }
      return this.getCollectionPropertiesValue(x, a.join("."));
    }
    return x;
  },


  // gets properties values from a given collection
  getCollectionPropertiesValue: function getCollectionPropertiesValue(collection, name, includeEmptyValues) {
    if (!name) {
      return collection;
    }
    if (typeof includeEmptyValues != "boolean") {
      includeEmptyValues = false;
    }
    var a = name.split("."),
        values = [];
    for (var i = 0, l = collection.length; i < l; i++) {
      var o = collection[i];

      if (!_utils2.default.has(o, a[0])) {
        if (includeEmptyValues) {
          values.push(null);
        }
        continue;
      }
      if (_utils2.default.isArray(o)) {
        var foundColl = this.getCollectionPropertiesValue(o, name);
        if (includeEmptyValues || foundColl.length) {
          values.push(foundColl);
        }
      } else if (_utils2.default.isPlainObject(o)) {
        var foundVal = this.getPropertyValue(o, name);
        if (includeEmptyValues || this.validateValue(foundVal)) {
          values.push(foundVal);
        }
      } else {
        if (includeEmptyValues || this.validateValue(o)) {
          values.push(o);
        }
      }
    }
    return values;
  },


  // returns true if the object has a significant value, false otherwise
  validateValue: function validateValue(o) {
    if (!o) return false;
    if (_utils2.default.isArray(o)) {
      return !!o.length;
    }
    return true;
  }
}; /**
    * Reflection utilities.
    * https://github.com/RobertoPrevato/KingTable
    *
    * Copyright 2018, Roberto Prevato
    * https://robertoprevato.github.io
    *
    * Licensed under the MIT license:
    * http://www.opensource.org/licenses/MIT
    */