"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * XML and HTML utilities to build HTML strings.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */

function isNum(x) {
  return typeof x == "number";
}

/**
 * A virtual XML element.
 */

var VXmlElement = function () {
  function VXmlElement(tagName, attributes, children) {
    _classCallCheck(this, VXmlElement);

    this.tagName = tagName;
    this.attributes = attributes || {};
    if (children && children instanceof Array == false) {
      // NB: children can be anything implementing a toString method (duck typing)
      children = [children];
    }
    this.children = children || [];
    this.hidden = false;
    this.empty = false;
  }

  _createClass(VXmlElement, [{
    key: "appendChild",
    value: function appendChild(child) {
      this.children.push(child);
    }

    /**
     * Converts this VXmlElement into an XML fragment.
     */

  }, {
    key: "toString",
    value: function toString(indent, indentChar, level) {
      var self = this,
          empty = self.empty,
          tagName = self.tagName,
          attrs = self.attributes,
          indent = isNum(indent) ? indent : 0,
          level = isNum(level) ? level : 0,
          indentString = indent > 0 ? new Array(indent * level + 1).join(indentChar || " ") : "",
          s = "<" + tagName,
          x;
      for (x in attrs) {
        if (attrs[x] !== undefined) {
          if (BOOLEAN_PROPERTIES.indexOf(x) > -1) {
            s += " " + x;
          } else {
            s += " " + x + "=\"" + attrs[x] + "\"";
          }
        }
      }
      if (empty) {
        s += " />";
        if (indent > 0) {
          // add the indent at the beginning of the string
          s = indentString + s + "\n";
        }
        return s;
      }
      s += ">";
      var children = self.children;
      if (indent > 0 && children.length) {
        s += "\n";
      }
      for (var i = 0, l = children.length; i < l; i++) {
        var child = children[i];
        if (!child) continue;
        // support HTML fragments
        if (typeof child == "string") {
          s += indentString + child + "\n";
        } else {
          if (!child.hidden) {
            s += child.toString(indent, indentChar, level + 1);
          }
        }
      }
      if (children && children.length) {
        s += indentString + ("</" + tagName + ">");
      } else {
        s += "</" + tagName + ">";
      }
      if (indent > 0) {
        // add the indent at the beginning of the string
        s = indentString + s + "\n";
      }
      return s;
    }
  }, {
    key: "tagName",
    get: function get() {
      return this._tagName;
    },
    set: function set(val) {
      if (typeof val != "string") throw new Error("tagName must be a string");
      if (!val.trim()) throw new Error("tagName must have a length");
      if (val.indexOf(" ") > -1) throw new Error("tagName cannot contain spaces");
      this._tagName = val;
    }
  }]);

  return VXmlElement;
}();

var EMPTY_ELEMENTS = "area base basefont br col frame hr img input isindex link meta param".split(" ");
var BOOLEAN_PROPERTIES = "checked selected disabled readonly multiple ismap isMap defer noresize noResize nowrap noWrap noshade noShade compact".split(" ");

function escapeHtml(unsafe) {
  return unsafe.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/"/g, "&quot;").replace(/'/g, "&#039;");
}

/**
 * A virtual HTML element.
 */

var VHtmlElement = function (_VXmlElement) {
  _inherits(VHtmlElement, _VXmlElement);

  function VHtmlElement(tagName, attributes, children) {
    _classCallCheck(this, VHtmlElement);

    var _this = _possibleConstructorReturn(this, (VHtmlElement.__proto__ || Object.getPrototypeOf(VHtmlElement)).call(this, tagName, attributes, children));

    _this.empty = EMPTY_ELEMENTS.indexOf(tagName.toLowerCase()) > -1;
    return _this;
  }

  _createClass(VHtmlElement, [{
    key: "id",
    get: function get() {
      return this.attributes.id;
    },
    set: function set(val) {
      this.attributes.id = val;
    }
  }]);

  return VHtmlElement;
}(VXmlElement);

/**
 * A virtual text element.
 */


var VTextElement = function () {
  function VTextElement(text) {
    _classCallCheck(this, VTextElement);

    this.text = text;
  }

  _createClass(VTextElement, [{
    key: "toString",
    value: function toString(indent, indentChar, level) {
      var indent = isNum(indent) ? indent : 0,
          level = isNum(level) ? level : 0,
          indentString = indent > 0 ? new Array(indent * level + 1).join(indentChar || " ") : "";
      return indentString + this.text + (indentString ? "\n" : "");
    }
  }, {
    key: "text",
    get: function get() {
      return this._text;
    },
    set: function set(val) {
      // escape characters that need to be escaped
      if (!val) val = "";
      if (typeof val != "string") val = val.toString();
      this._text = escapeHtml(val);
    }
  }]);

  return VTextElement;
}();

/**
 * A piece of HTML fragment that should be rendered without escaping.
 */


var VHtmlFragment = function () {
  function VHtmlFragment(html) {
    _classCallCheck(this, VHtmlFragment);

    this.html = html;
  }

  _createClass(VHtmlFragment, [{
    key: "toString",
    value: function toString(indent, indentChar, level) {
      var indent = isNum(indent) ? indent : 0,
          level = isNum(level) ? level : 0,
          indentString = indent > 0 ? new Array(indent * level + 1).join(indentChar || " ") : "";
      return indentString + this.html + (indentString ? "\n" : "");
    }
  }]);

  return VHtmlFragment;
}();

/**
 * A virtual comment element.
 */


var VCommentElement = function () {
  function VCommentElement(text) {
    _classCallCheck(this, VCommentElement);

    this.text = text;
  }

  _createClass(VCommentElement, [{
    key: "toString",
    value: function toString(indent, indentChar, level) {
      var indent = isNum(indent) ? indent : 0,
          level = isNum(level) ? level : 0,
          indentString = indent > 0 ? new Array(indent * level + 1).join(indentChar || " ") : "";
      return indentString + "<!--" + this.text + "-->" + (indentString ? "\n" : "");
    }
  }, {
    key: "text",
    get: function get() {
      return this._text;
    },
    set: function set(val) {
      if (!val) val = "";
      if (typeof val != "string") val = val.toString();
      // disallows <!-- and --> inside the comment text
      val = val.replace(/<!--/g, "").replace(/-->/g, "");
      this._text = val;
    }
  }]);

  return VCommentElement;
}();

/**
 * Virtual wrapper element for multiple elements without single root.
 */


var VWrapperElement = function () {
  function VWrapperElement(children) {
    _classCallCheck(this, VWrapperElement);

    this.children = children;
    this.hidden = false;
  }

  _createClass(VWrapperElement, [{
    key: "toString",
    value: function toString(indent, indentChar, level) {
      var s = "",
          children = this.children;
      if (!children || this.hidden) return s;

      for (var i = 0, l = children.length; i < l; i++) {
        var child = children[i];
        if (!child) continue;
        if (!child.hidden) {
          s += child.toString(indent, indentChar, level);
        }
      }
      return s;
    }
  }]);

  return VWrapperElement;
}();

exports.VXmlElement = VXmlElement;
exports.VHtmlElement = VHtmlElement;
exports.VHtmlFragment = VHtmlFragment;
exports.VTextElement = VTextElement;
exports.VCommentElement = VCommentElement;
exports.VWrapperElement = VWrapperElement;
exports.escapeHtml = escapeHtml;