"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _utils = require("../../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _date = require("../../scripts/components/date");

var _date2 = _interopRequireDefault(_date);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Proxy functions for built-in JSON API.
 * Proxy functions are used, for example, to remove the asymmetry between
 * -  JSON stringify (creating string representations of dates) and
 * -  JSON parse (NOT parsing dates in ISO format - losing dates).
 *
 * Besides, I HATE the word "stringify"!!!
 *
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
exports.default = {
  /**
   * Serializes an object into JSON format.
   */
  compose: function compose(o, indentation) {
    return JSON.stringify(o, function (k, v) {
      if (v === undefined) {
        return null;
      }
      return v;
    }, indentation);
  },

  /**
   * Parses an object represented in JSON format.
   */
  parse: function parse(s, options) {
    var o = _utils2.default.extend({
      parseDates: true
    }, options);

    if (!o.parseDates) {
      return JSON.parse(s);
    }

    return JSON.parse(s, function (k, v) {
      if (_utils2.default.isString(v) && _date2.default.looksLikeDate(v)) {
        // check if the value looks like a date and can be parsed
        var a = _date2.default.parse(v);
        if (a && _date2.default.isValid(a)) {
          return a;
        }
      }
      return v;
    });
  },

  /**
   * Clones an object using JSON.
   * Unlike the normal JSON API, Dates are kept as dates;
   * however, strings that looks like dates becomes dates.
   */
  clone: function clone(o) {
    return this.parse(this.compose(o));
  }
};