"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require("../../scripts/components/events");

var _events2 = _interopRequireDefault(_events);

var _utils = require("../../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _raise = require("../../scripts/raise");

var _raise2 = _interopRequireDefault(_raise);

var _regex = require("../../scripts/components/regex");

var _regex2 = _interopRequireDefault(_regex);

var _array = require("../../scripts/components/array");

var _array2 = _interopRequireDefault(_array);

var _string = require("../../scripts/components/string");

var _string2 = _interopRequireDefault(_string);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Filters manager.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Provides methods to handle client side filtering logic for arrays.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * https://github.com/RobertoPrevato/KingTable
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2018, Roberto Prevato
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * https://robertoprevato.github.io
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Licensed under the MIT license:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * http://www.opensource.org/licenses/MIT
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var FiltersManager = function (_EventsEmitter) {
  _inherits(FiltersManager, _EventsEmitter);

  function FiltersManager(options, staticProperties) {
    var _ret;

    _classCallCheck(this, FiltersManager);

    var _this = _possibleConstructorReturn(this, (FiltersManager.__proto__ || Object.getPrototypeOf(FiltersManager)).call(this));

    _this.rules = [];
    _this.searchDisabled = false;
    _this.init(options, staticProperties);
    return _ret = _this, _possibleConstructorReturn(_this, _ret);
  }

  _createClass(FiltersManager, [{
    key: "init",
    value: function init(options, staticProperties) {
      if (staticProperties) _utils2.default.extend(this, staticProperties);
      this.options = _utils2.default.extend({}, this.defaults, options);
    }
  }, {
    key: "set",
    value: function set(filter, options) {
      options = _utils2.default.extend({
        silent: false
      }, options || {});
      if (!filter) return this;
      if (filter.id && !filter.key) filter.key = filter.id;
      if (filter.key) {
        this.rules = _utils2.default.reject(this.rules, function (r) {
          return r.key === filter.key;
        });
      }
      if (filter.fromLiveFilters) return this.setLiveFilter(filter);
      this.rules.push(filter);
      if (!options.silent) {
        this.onRulesChange(filter);
      }
      return this;
    }
  }, {
    key: "setLiveFilter",
    value: function setLiveFilter(filter) {
      (0, _raise2.default)(12, "LiveFilter feature not implemented.");
    }
  }, {
    key: "getRuleByKey",


    /**
     * Gets a rule by key.
     */
    value: function getRuleByKey(key) {
      return _utils2.default.find(this.rules, function (rule) {
        return rule.key == key;
      });
    }

    /**
     * Get all rules by type.
     */

  }, {
    key: "getRulesByType",
    value: function getRulesByType(type) {
      return _utils2.default.where(this.rules, function (rule) {
        return rule.type == type;
      });
    }

    /**
     * Removes a single rule by key.
     */

  }, {
    key: "removeRuleByKey",
    value: function removeRuleByKey(key, options) {
      options = _utils2.default.extend({
        silent: false
      }, options || {});
      var self = this,
          rules = self.rules;
      var ruleToRemove = _utils2.default.find(rules, function (r) {
        return r.key == key;
      });
      if (ruleToRemove) {
        self.rules = _utils2.default.reject(rules, function (r) {
          return r === ruleToRemove;
        });
        if (!options.silent) {
          self.onRulesChange();
        }
      }
      return self;
    }

    /**
     * Function to fire when rules change.
     * Extensibility point.
     */

  }, {
    key: "onRulesChange",
    value: function onRulesChange() {}

    /**
     * Searches inside a collection for all items that respect a given search string.
     */

  }, {
    key: "search",
    value: function search(collection, s, options) {
      if (!s || !collection || this.searchDisabled) return collection;
      var rx = s instanceof RegExp ? s : _regex2.default.getSearchPattern(_string2.default.getString(s), options);
      if (!rx) return false;
      if (!options.searchProperties)
        // try to get search properties from the context
        options.searchProperties = this.context.getSearchProperties();
      if (!options.searchProperties) (0, _raise2.default)(11, "missing search properties");
      return _array2.default.searchByStringProperties({
        pattern: rx,
        properties: options.searchProperties,
        collection: collection,
        keepSearchDetails: false
      });
    }

    /**
     * Skims an array, applying all configured filters.
     */

  }, {
    key: "skim",
    value: function skim(arr) {
      var self = this,
          rules = self.rules,
          l = rules.length;
      if (!l) return arr;
      var a = arr;
      for (var i = 0; i < l; i++) {
        var filter = self.rules[i];
        if (filter.disabled) continue;
        a = self.applyFilter(a, filter);
      }
      return a;
    }

    /**
     * Applies a given filter to an array.
     */

  }, {
    key: "applyFilter",
    value: function applyFilter(arr, filter) {
      switch (filter.type) {
        case "search":
          return this.search(arr, filter.value, filter);
        case "fn":
        case "function":
          return _utils2.default.where(arr, _utils2.default.partial(filter.fn.bind(filter.context || this), filter));
      }
      return arr;
    }

    /**
     * Resets this FiltersManager, removing all filter rules in it.
     */

  }, {
    key: "reset",
    value: function reset() {
      var rule;
      while (rule = this.rules.shift()) {
        if (rule.onReset) {
          rule.onReset.call(this);
        }
      }
      return this;
    }
  }], [{
    key: "defaults",
    get: function get() {
      return {};
    }
  }]);

  return FiltersManager;
}(_events2.default);

exports.default = FiltersManager;