"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Paginator class.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Offers methods to handle pagination of items and page number.s
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * https://github.com/RobertoPrevato/KingTable
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright 2018, Roberto Prevato
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * https://robertoprevato.github.io
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Licensed under the MIT license:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * http://www.opensource.org/licenses/MIT
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _utils = require("../../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function checkNumber() {
  var i,
      l = arguments.length,
      a;
  for (i = 0; i < l; i++) {
    a = arguments[i];
    if (!_utils2.default.isNumber(a)) throw new Error("invalid type");
  }
}
function checkNullableNumber() {
  var i,
      l = arguments.length,
      a;
  for (i = 0; i < l; i++) {
    a = arguments[i];
    if (!_utils2.default.isUnd(a) && !_utils2.default.isNumber(a)) throw new Error("invalid type");
  }
}

var Paginator = function () {
  function Paginator(options) {
    _classCallCheck(this, Paginator);

    options = options || {};
    checkNullableNumber(options.page, options.totalItemsCount, options.resultsPerPage);
    this.page = options.page || 0;
    this.resultsPerPage = options.resultsPerPage || 30;
    this.totalItemsCount = options.totalItemsCount || Infinity;
    this.totalPageCount = Infinity;
    this.firstObjectNumber = undefined;
    this.lastObjectNumber = undefined;
    if (!_utils2.default.isUnd(options.totalItemsCount)) {
      this.setTotalItemsCount(options.totalItemsCount, true);
    }
    if (_utils2.default.isFunction(options.onPageChange)) {
      // override
      this.onPageChange = options.onPageChange;
    }
  }

  _createClass(Paginator, [{
    key: "data",


    /**
     * Returns a summary of this paginator.
     */
    value: function data() {
      var data = this;
      return {
        page: data._page,
        totalPageCount: data.totalPageCount,
        resultsPerPage: data.resultsPerPage,
        firstObjectNumber: data.firstObjectNumber,
        lastObjectNumber: data.lastObjectNumber,
        totalItemsCount: data.totalItemsCount
      };
    }

    /**
     * Goes to previous page.
     */

  }, {
    key: "prev",
    value: function prev() {
      var self = this,
          a = self.page - 1;
      if (self.validPage(a)) {
        self.page = a;
      }
      return self;
    }

    /**
     * Goes to next page.
     */

  }, {
    key: "next",
    value: function next() {
      var self = this,
          a = self.page + 1;
      if (self.validPage(a)) {
        self.page = a;
      }
      return self;
    }

    /**
     * Goes to the first page.
     */

  }, {
    key: "first",
    value: function first() {
      this.page = 1;
      return this;
    }

    /**
     * Goes to the last page.
     */

  }, {
    key: "last",
    value: function last() {
      this.page = this.totalPageCount;
      return this;
    }

    /**
     * Updates the objects numbers in memory.
     */

  }, {
    key: "updateItemsNumber",
    value: function updateItemsNumber() {
      var self = this,
          totalItemsCount = self.totalItemsCount;
      self.firstObjectNumber = self.page * self.resultsPerPage - self.resultsPerPage + 1;
      self.lastObjectNumber = Math.min(_utils2.default.isNumber(totalItemsCount) ? totalItemsCount : Infinity, self.page * self.resultsPerPage);
    }

    /**
     * Sets the total items count of this paginator.
     */

  }, {
    key: "setTotalItemsCount",
    value: function setTotalItemsCount(itemsCount, uponInitialization) {
      checkNumber(itemsCount);
      var self = this;
      self.totalItemsCount = itemsCount;
      var totalPages = self.getPageCount(itemsCount, self.resultsPerPage);
      self.totalPageCount = totalPages;
      //if the current page is greater than the total pages count; set automatically the page to 1
      // NB: the following does not make sense upon initialization!!
      if (!uponInitialization && totalPages < self.page) {
        self.page = 1;
      }
      self.updateItemsNumber();
      return self;
    }

    /**
     * Returns a value indicating whether the given value is a valid page number for this paginator.
     *
     * @param val: page number
     */

  }, {
    key: "validPage",
    value: function validPage(val) {
      var p = this;
      return !(isNaN(val) || val < 1 || val > p.totalPageCount || val === p.page);
    }

    /**
     * Function fired when changing page.
     */

  }, {
    key: "onPageChange",
    value: function onPageChange() {}

    /**
     * Gets the total page count to display n objects, given the number of objects per page.
     *
     * @param objectsCount: total items count
     * @param objectsPerPage: page size, number of items per page
     */

  }, {
    key: "getPageCount",
    value: function getPageCount(objectsCount, objectsPerPage) {
      checkNumber(objectsCount, objectsPerPage);
      if (objectsCount === Infinity) return Infinity;
      if (objectsCount === -Infinity) return 0;
      if (objectsCount < 1) return 0;
      if (objectsCount > objectsPerPage) {
        if (objectsCount % objectsPerPage == 0) {
          return objectsCount / objectsPerPage;
        }
        return Math.ceil(objectsCount / objectsPerPage);
      }
      return 1;
    }
  }, {
    key: "dispose",
    value: function dispose() {
      delete this.onPageChange;
    }
  }, {
    key: "resultsPerPage",
    get: function get() {
      return this._resultsPerPage;
    },
    set: function set(value) {
      if (!value) value = 0;
      checkNumber(value);
      var self = this,
          totalItemsCount = self.totalItemsCount;
      // is total items count known?
      if (totalItemsCount) {
        var totalPages = self.getPageCount(totalItemsCount, value);
        self.totalPageCount = totalPages;
        if (totalPages <= self._page) {
          // go to last page
          self.page = totalPages;
        }
      }
      self._resultsPerPage = value;
      self.updateItemsNumber();
    }

    /**
     * Gets the current page of this paginator.
     */

  }, {
    key: "page",
    get: function get() {
      return this._page;
    }

    /**
     * Sets the current page of this paginator.
     */
    ,
    set: function set(value) {
      checkNumber(value);
      if (value != this.page) {
        this._page = value;
        this.updateItemsNumber();
        this.onPageChange();
      }
    }
  }]);

  return Paginator;
}();

exports.default = Paginator;