"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.menuItemBuilder = exports.menuBuilder = undefined;

var _utils = require("../../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _dom = require("../../scripts/dom");

var _dom2 = _interopRequireDefault(_dom);

var _exceptions = require("../../scripts/exceptions");

var _html = require("../../scripts/data/html");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * KingTable menu builder function.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
function buildMenuItemCaret() {
  return new _html.VHtmlElement("span", {
    "class": "oi",
    "data-glyph": "caret-right"
  });
}

function menuBuilder(menus) {
  if (!menus) throw "missing menus";
  if (_utils2.default.isPlainObject(menus)) return menuBuilder([menus]);
  if (!_utils2.default.isArray(menus) || !menus.length) throw "missing menus";
  //normalize schema, if needed
  var first = menus[0];
  if (!first.items && first.menu) {
    menus = [{ items: menus }];
  }
  var a = _utils2.default.map(menus, function (menu) {
    var items = menu.items;
    return new _html.VHtmlElement("ul", {
      "id": menu.id,
      "class": "ug-menu"
    }, items ? _utils2.default.map(items, function (x) {
      if (!x) return;
      return menuItemBuilder(x);
    }) : null);
  });
  return new _html.VWrapperElement(a);
}

function menuItemCaret() {
  return new _html.VHtmlElement("span", {
    "class": "oi",
    "data-glyph": "caret-right"
  });
}

function menuItemBuilder(options) {
  var o = options || {};
  var type = o.type,
      href = o.href,
      classes = [],
      name = o.name,
      submenu = o.menu,
      attr = o.attr,
      caret = submenu ? buildMenuItemCaret() : null,
      children = [],
      el,
      nameTextEl = new _html.VTextElement(name || "");
  if (attr && attr.css && !attr["class"]) {
    // allow to use attribute css for class
    attr["class"] = attr.css;
    delete attr.css;
  }
  switch (type) {
    case "checkbox":
      var cid = _utils2.default.uniqueId("mnck-");
      var checked = o.checked ? true : undefined;
      el = new _html.VWrapperElement([new _html.VHtmlElement("input", _utils2.default.extend({}, attr, {
        "id": cid,
        "type": "checkbox",
        "checked": checked
      })), new _html.VHtmlElement("label", {
        "for": cid
      }, nameTextEl)]);
      break;
    case "radio":
      var value = o.value;
      if (!value) throw new Error("missing 'value' for radio menu item");
      var cid = _utils2.default.uniqueId("mnrd-");
      var checked = o.checked ? true : undefined;
      el = new _html.VWrapperElement([new _html.VHtmlElement("input", _utils2.default.extend({}, attr, {
        "id": cid,
        "type": "radio",
        "checked": checked,
        "value": value
      })), new _html.VHtmlElement("label", {
        "for": cid
      }, nameTextEl)]);
      break;
    default:
      if (href) {
        el = new _html.VHtmlElement("a", _utils2.default.extend({
          "href": href
        }, attr), [nameTextEl, caret]);
      } else {
        el = new _html.VHtmlElement("span", _utils2.default.extend({
          "tabindex": "0"
        }, attr), [nameTextEl, caret]);
      }
      break;
  }
  // name element
  children.push(el);

  if (submenu) {
    children.push(menuBuilder(submenu));
  }

  return new _html.VHtmlElement("li", {
    "id": o.id,
    "class": submenu ? "ug-submenu" : undefined
  }, children);
}

exports.menuBuilder = menuBuilder;
exports.menuItemBuilder = menuItemBuilder;