"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _dom = require("../../scripts/dom");

var _dom2 = _interopRequireDefault(_dom);

var _utils = require("../../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * KingTable menu core functions.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
function eventToIgnore(e) {
  return (/input|select|textarea|label|^a$/i.test(e.target.tagName)
  );
}

var menufunctions = {

  closeMenus: function closeMenus(e) {
    var self = this;
    if (e && e.which === 3) return;
    _utils2.default.each(["ug-menu", "ug-submenu"], function (className) {
      var elements = document.body.getElementsByClassName(className);

      _utils2.default.each(elements, function (el) {
        if (_dom2.default.contains(el, e.target)) return;

        var parent = el.parentNode;
        if (!_dom2.default.hasClass(parent, "open")) return;

        if (/input|textarea/i.test(e.target.tagName) && _dom2.default.contains(parent, e.target)) return;

        _dom2.default.removeClass(parent, "open");
      });
    });
  },

  expandMenu: function expandMenu(e) {
    if (eventToIgnore(e)) return true;
    var self = this,
        el = e.target,
        disabled = "disabled";
    if (_dom2.default.hasClass(el, disabled) || el.hasAttribute(disabled)) {
      return false;
    }
    var parent = el.parentElement,
        open = "open";
    if (_dom2.default.hasClass(parent, open)) {
      _dom2.default.removeClass(parent, open);
    } else {
      _dom2.default.addClass(parent, open);
    }
    e.preventDefault();
    return false;
  },

  expandSubMenu: function expandSubMenu(e) {
    if (eventToIgnore(e)) return true;
    var open = "open",
        el = _dom2.default.closestWithTag(e.target, "li"),
        siblings = _dom2.default.siblings(el);
    _utils2.default.each(siblings, function (sib) {
      _dom2.default.removeClass(sib, open);
      var allOpen = sib.getElementsByClassName(open);
      _utils2.default.each(allOpen, function (a) {
        _dom2.default.removeClass(a, open);
      });
    });
    _dom2.default.addClass(el, open);
    return false;
  }
};

exports.default = {
  setup: function setup() {
    if (this.initialized) {
      return false;
    }
    this.initialized = true;
    var click = "click.menus",
        keydown = "keydown.menus",
        bo = document.body;
    _dom2.default.off(bo, click);
    _dom2.default.on(bo, click, menufunctions.closeMenus); // order is important
    _dom2.default.on(bo, click, ".ug-expander", menufunctions.expandMenu);
    _dom2.default.on(bo, click, ".ug-submenu", menufunctions.expandSubMenu);
  }
};