"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /**
                                                                                                                                                                                                                                                                               * KingTable core class.
                                                                                                                                                                                                                                                                               * This class is responsible of fetching data, handling responses,
                                                                                                                                                                                                                                                                               * configuring columns.
                                                                                                                                                                                                                                                                               * https://github.com/RobertoPrevato/KingTable
                                                                                                                                                                                                                                                                               *
                                                                                                                                                                                                                                                                               * Copyright 2018, Roberto Prevato
                                                                                                                                                                                                                                                                               * https://robertoprevato.github.io
                                                                                                                                                                                                                                                                               *
                                                                                                                                                                                                                                                                               * Licensed under the MIT license:
                                                                                                                                                                                                                                                                               * http://www.opensource.org/licenses/MIT
                                                                                                                                                                                                                                                                               */


var _kingtableText = require("../../scripts/tables/kingtable.text.builder");

var _kingtableText2 = _interopRequireDefault(_kingtableText);

var _kingtableHtml = require("../../scripts/tables/kingtable.html.builder");

var _kingtableHtml2 = _interopRequireDefault(_kingtableHtml);

var _kingtableHtmlBase = require("../../scripts/tables/kingtable.html.base.builder");

var _kingtableHtmlBase2 = _interopRequireDefault(_kingtableHtmlBase);

var _kingtableRhtml = require("../../scripts/tables/kingtable.rhtml.builder");

var _kingtableRhtml2 = _interopRequireDefault(_kingtableRhtml);

var _kingtable = require("../../scripts/tables/kingtable.regional");

var _kingtable2 = _interopRequireDefault(_kingtable);

var _events = require("../../scripts/components/events");

var _events2 = _interopRequireDefault(_events);

var _objectAnalyzer = require("../../scripts/data/object-analyzer");

var _objectAnalyzer2 = _interopRequireDefault(_objectAnalyzer);

var _sanitizer = require("../../scripts/data/sanitizer");

var _sanitizer2 = _interopRequireDefault(_sanitizer);

var _filtersManager = require("../../scripts/filters/filters-manager");

var _filtersManager2 = _interopRequireDefault(_filtersManager);

var _paginator = require("../../scripts/filters/paginator");

var _paginator2 = _interopRequireDefault(_paginator);

var _ajax = require("../../scripts/data/ajax");

var _ajax2 = _interopRequireDefault(_ajax);

var _raise = require("../../scripts/raise");

var _raise2 = _interopRequireDefault(_raise);

var _utils = require("../../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _string = require("../../scripts/components/string");

var _string2 = _interopRequireDefault(_string);

var _regex = require("../../scripts/components/regex");

var _regex2 = _interopRequireDefault(_regex);

var _number = require("../../scripts/components/number");

var _number2 = _interopRequireDefault(_number);

var _date = require("../../scripts/components/date");

var _date2 = _interopRequireDefault(_date);

var _array = require("../../scripts/components/array");

var _array2 = _interopRequireDefault(_array);

var _csv = require("../../scripts/data/csv");

var _csv2 = _interopRequireDefault(_csv);

var _json = require("../../scripts/data/json");

var _json2 = _interopRequireDefault(_json);

var _xml = require("../../scripts/data/xml");

var _xml2 = _interopRequireDefault(_xml);

var _file = require("../../scripts/data/file");

var _file2 = _interopRequireDefault(_file);

var _lru = require("../../scripts/data/lru");

var _lru2 = _interopRequireDefault(_lru);

var _memstore = require("../../scripts/data/memstore");

var _memstore2 = _interopRequireDefault(_memstore);

var _exceptions = require("../../scripts/exceptions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var VERSION = "2.0.0";

var DEFAULTS = {

  // Table language.
  lang: "en",

  // Table caption.
  caption: null,

  // Whether to display the item number or not.
  itemCount: true,

  // Default schema for each table column.
  columnDefault: {
    name: "",
    type: "text",
    sortable: true,
    allowSearch: true,
    hidden: false
    // secret: undefined
    // format: undefined (allows to define formatting function) 
  },

  httpMethod: "GET", // method to use to fetch data, when using AJAX requests

  // Whether to allow search, or not.
  allowSearch: true,

  // Minimum number of characters inside the search field to trigger a search.
  minSearchChars: 3,

  // Default first page.
  page: 1,

  // Default page size
  resultsPerPage: 30,

  // Suffix to use for formatted properties
  formattedSuffix: "_(formatted)",

  // Permits to specify whether the collection is fixed or not
  // default changes if the table is instantiated passing a collection
  fixed: undefined,

  // Permits to specify an initial search when generating the table for the first time
  search: "",

  // Permits to specify how sort by information must be transmitted
  sortByFormatter: _array2.default.humanSortBy,

  // Permits to specify the search mode to use during live search
  // FullString, SplitWords or SplitSentences
  searchMode: "FullString",

  // Default export formats.
  exportFormats: [{
    name: "Csv",
    format: "csv",
    type: "text/csv",
    cs: true // client side
  }, {
    name: "Json",
    format: "json",
    type: "application/json",
    cs: true // client side
  }, {
    name: "Xml",
    format: "xml",
    type: "text/xml",
    cs: true // client side
  }],

  // Whether to prettify xml when exporting, or not.
  prettyXml: true,

  // Allows to specify csv serialization options
  csvOptions: {},

  // Whether to include hidden properties in the export; or not.
  exportHiddenProperties: false,

  // Kind of builder.
  builder: "rhtml",

  // Whether to store data returned by `getTableData` function, or not.
  // If true, data is stored in memory and in data storage for later use.
  storeTableData: true,

  // The LRU cache size (how many items per key can be stored).
  lruCacheSize: 10,

  // The LRU cache max age in milliseconds - default 15 minutes; (<= 0 for no expiration).
  lruCacheMaxAge: 60 * 1e3 * 15,

  // Whether the anchor timestamp should be shown or not
  showAnchorTimestamp: true,

  collectionName: "data",

  // When text search is used, its sort logic takes precedence over the sort criteria defined clicking on columns.
  searchSortingRules: true,

  // The name of the property that should be used as id.
  idProperty: null,

  // Whether searched values should be automatically highlighted
  autoHighlightSearchProperties: true,

  // Value to use for represent null or empty values.
  emptyValue: ""
};

var BUILDERS = {
  "text": _kingtableText2.default,
  "html": _kingtableHtml2.default,
  "rhtml": _kingtableRhtml2.default
};

var UNDEFINED = "undefined";

if ((typeof Promise === "undefined" ? "undefined" : _typeof(Promise)) == UNDEFINED) {
  var fixed = false;
  // check if ES6Promise was loaded
  if ((typeof ES6Promise === "undefined" ? "undefined" : _typeof(ES6Promise)) != UNDEFINED && ES6Promise.polyfill) {
    try {
      ES6Promise.polyfill();
      fixed = (typeof Promise === "undefined" ? "undefined" : _typeof(Promise)) != UNDEFINED;
    } catch (ex) {
      // ignore
    }
  }
  if (!fixed) {
    (0, _raise2.default)(1, "Missing implementation of Promise (missing dependency)");
  }
}

var KingTable = function (_EventsEmitter) {
  _inherits(KingTable, _EventsEmitter);

  /**
   * Creates a new instance of KingTable with the given options and static properties.
   *
   * @param options: options to use for this instance of KingTable.
   * @param staticProperties: properties to override in the instance of KingTable.
   */
  function KingTable(options, staticProperties) {
    _classCallCheck(this, KingTable);

    var _this = _possibleConstructorReturn(this, (KingTable.__proto__ || Object.getPrototypeOf(KingTable)).call(this));

    options = options || {};
    var self = _this;
    //
    // The second argument allows to override properties of the KingTable instance.
    //
    if (staticProperties) {
      _utils2.default.extend(self, staticProperties);
    }
    //
    // Base properties are automatically set from the first argument into the instance
    //
    _utils2.default.each(self.baseProperties(), function (x) {
      if (_utils2.default.has(options, x)) {
        self[x] = options[x];
        delete options[x];
      }
    });
    var sortBy = options.sortBy;
    if (_utils2.default.isString(sortBy)) {
      self.sortCriteria = _array2.default.getSortCriteria(sortBy);
    }

    // Set options
    options = self.options = _utils2.default.extend({}, KingTable.defaults, options);
    self.loading = false;
    self.init(options, staticProperties);
    return _this;
  }

  /**
   * An array of property names, that can be overridden using the first argument (options) of KingTable constructor,
   * instead of using the second argument, which allows to override any property in an instance of KingTable.
   * Properties outside of this array and passed as first argument, are instead put in the "options" property of a KingTable.
   */


  _createClass(KingTable, [{
    key: "baseProperties",
    value: function baseProperties() {
      return ["id", // allows to set an id to the kingtable
      "onInit", // function to execute after initialization
      "element", // allows to specify the table element
      "context", // table context
      "fixed", //
      "prepareData", //
      "getExtraFilters", //
      "getTableData", // function to get required data to render the table
      "afterRender", // function to execute after render
      "beforeRender", // function to execute before render
      "numberFilterFormatter", // function to format numbers for filters
      "dateFilterFormatter" // function to format dates for filters
      ];
    }
  }, {
    key: "init",


    /**
     * Initializes an instance of KingTable.
     */
    value: function init(options) {
      var self = this;
      self.cache = {};
      self.disposables = [];
      self.analyst = new _objectAnalyzer2.default();
      self.sanitizer = new _sanitizer2.default();
      self.filters = new _filtersManager2.default({}, {
        context: self
      });

      var data = options.data;
      if (data) {
        if (!_utils2.default.isArray(data)) (0, _raise2.default)(3, "Data is not an array");
        // KingTable initialized passing an array of items,
        // unless specified otherwise, assume that it is a fixed table
        // (i.e. a table that doesn't require server side pagination)

        // Apply the same logic that would be applied if data was coming from
        // server side (e.g. parsing of dates)
        data = _json2.default.clone(data);
        self.setFixedData(data);
      }
      // NB: it is important to load settings after setting data;
      // because if client side search is used, search filter requires search properties
      self.loadSettings(); // load settings from storage, if applicable
      self.setPagination();

      if (!self.fixed) {
        // if the table collection is not fixed;
        // then there is no need to perform search operations on the client side;
        self.filters.searchDisabled = true;
      }

      // initialize the current builder
      // this is required to allow displaying loading information
      self.setBuilder(options.builder).onInit();
      return self;
    }

    /**
     * Returns the translations for the current language configuration.
     */

  }, {
    key: "getReg",
    value: function getReg() {
      var lang = this.options.lang;
      if (!lang) (0, _raise2.default)(15, "Missing language option (cannot be null or falsy)");
      var o = KingTable.regional[lang];
      if (!o) (0, _raise2.default)(15, "Missing regional for language " + lang);
      return o;
    }

    /**
     * Sets the builder handler.
     *
     * @param {string} name: builder type.
     */

  }, {
    key: "setBuilder",
    value: function setBuilder(name) {
      if (!name) (0, _raise2.default)(8, "name cannot be null or empty");
      var self = this,
          o = self.options,
          builders = KingTable.builders;
      if (self.builder) {
        self.disposeOf(self.builder);
      }
      var builderType = builders[name];
      if (!builderType) {
        (0, _raise2.default)(10, "Missing handler for builder: " + name);
      }
      var builder = new builderType(self);
      self.builder = builder;
      self.disposables.push(builder);
      return self;
    }

    /**
     * Applies client side pagination to an array, on the basis of current configuration.
     */

  }, {
    key: "getSubSet",
    value: function getSubSet(a) {
      var pagination = this.pagination;
      var from = (pagination.page - 1) * pagination.resultsPerPage,
          to = pagination.resultsPerPage + from;
      return a.slice(from, to);
    }

    /**
     * Sets the pagination data inside the instance of KingTable.
     *
     * @returns {KingTable}
     */

  }, {
    key: "setPagination",
    value: function setPagination() {
      var self = this,
          data = self.data,
          options = self.options,
          page = options.page,
          resultsPerPage = options.resultsPerPage,
          totalItemsCount = options.totalItemsCount || (data ? data.length : 0);
      if (self.pagination) {
        self.disposeOf(self.pagination);
      }
      var pagination = self.pagination = new _paginator2.default({
        page: page,
        resultsPerPage: resultsPerPage,
        totalItemsCount: totalItemsCount,
        onPageChange: function onPageChange() {
          self.render();
        }
      });
      self.disposables.push(pagination);
      return self;
    }

    /**
     * Updates the pagination, according to the total count of items
     * that satisfy the current filters.
     *
     * @param totalItemsCount: the total count of items that satisfy the current filters (except page number)
     * @returns {KingTable}
     */

  }, {
    key: "updatePagination",
    value: function updatePagination(totalItemsCount) {
      var self = this;
      if (!self.pagination) self.setPagination();
      if (!_utils2.default.isNumber(totalItemsCount)) throw "invalid type";
      var pagination = self.pagination;
      pagination.setTotalItemsCount(totalItemsCount);
      // results count change
      _utils2.default.ifcall(self.onResultsCountChange, self);
      self.trigger("change:pagination");
      return self;
    }

    /**
     * Function called when initialization is completed.
     * Extensibility point.
     */

  }, {
    key: "onInit",
    value: function onInit() {}

    /**
     * Returns a value indicating whether this instance of KingTable has data or
     * not.
     */

  }, {
    key: "hasData",
    value: function hasData() {
      var data = this.data;
      return !!(data && data.length);
    }

    /**
     * Returns the structure of the collection items.
     * By default, it is assumed that all items inside the collection have the
     * same structure.
     */

  }, {
    key: "getItemStructure",
    value: function getItemStructure() {
      // analyze whole collection, if necessary, only the first item if possible
      // if necessary == if any property is nullable and it's necessary to check
      // multiple items until a certain value is found (e.g. nullable numbers, dates)
      return this.analyst.describe(this.data, { lazy: true });
    }

    /**
     * Initializes the columns information for this KingTable.
     */

  }, {
    key: "initColumns",
    value: function initColumns() {
      var n = "columnsInitialized",
          self = this;
      if (self[n] || !self.hasData()) return self;
      self[n] = true;
      var columns = [];

      // gets the first object of the table as starting point
      var objSchema = self.getItemStructure();
      var x,
          properties = [];
      for (x in objSchema) {
        objSchema[x] = { name: x, type: objSchema[x] };
        properties.push(x);
      }
      var optionsColumns = self.options.columns;

      // does the user specified columns in constructor options?
      if (optionsColumns) {
        var i = 0,
            name;
        for (x in optionsColumns) {
          var col = optionsColumns[x];
          if (_utils2.default.isPlainObject(optionsColumns)) {
            name = x;
          } else if (_utils2.default.isArray(optionsColumns)) {
            if (_utils2.default.isString(col)) {
              (0, _raise2.default)(16, "invalid columns option " + col);
            }
            name = col.name;
            if (!name) {
              (0, _raise2.default)(17, "missing name in column option");
            }
          } else {
            (0, _raise2.default)(16, "invalid columns option");
          }
          // is the name inside the object schema?
          if (_utils2.default.indexOf(properties, name) == -1) {
            (0, _raise2.default)(18, "A column is defined with name \"" + name + "\", but this property was not found among object properties. Items properties are: " + properties.join(', '));
          }

          // support defining only the columns by their display name (to save programmers's time)
          if (_utils2.default.isString(col)) {
            // normalize
            col = optionsColumns[x] = { displayName: col };
          }
          if (_utils2.default.isString(col.name)) {
            // if a name property is defined, replace it with the key `displayName`,
            // since the KingTable requires the name to be equal to the actual property name.
            col.displayName = col.name;
            delete col.name;
          }
          objSchema[name] = _utils2.default.extend(objSchema[name], col);

          var colPos = col.position;
          if (_utils2.default.isNumber(colPos)) {
            objSchema[name].position = colPos;
          } else {
            objSchema[name].position = i;
          }
          i++;
        }
      }

      for (x in objSchema) {
        var base = { name: x },
            schema = objSchema[x],
            type = schema.type;
        if (!type) schema.type = type = "string";
        // extend with table column default options
        var col = _utils2.default.extend({}, self.options.columnDefault, base, schema);
        // assign a unique id to this column object:
        col.cid = _utils2.default.uniqueId("col");
        type = _utils2.default.lower(type);
        //set default properties by field type
        var a = KingTable.Schemas.DefaultByType;
        if (_utils2.default.has(a, type)) {
          //default schema by type
          var def = a[type];
          if (_utils2.default.isFunction(def)) def = def.call(self, schema, objSchema);
          _utils2.default.extend(base, def);
        }
        //set default properties by name
        a = KingTable.Schemas.DefaultByName;
        if (_utils2.default.has(a, x)) {
          //default schema by name
          _utils2.default.extend(base, a[x]);
        }

        _utils2.default.extend(col, base);

        if (optionsColumns) {
          //the user esplicitly defined some column options
          //columns are defined in the options, so take their defaults, supporting both arrays or plain objects
          var definedSchema = _utils2.default.isArray(optionsColumns) ? _utils2.default.find(optionsColumns, function (o) {
            return o.name == x;
          }) : optionsColumns[x];
          if (definedSchema) {
            //some options are explicitly defined for a field: extend existing schema with column defaults
            _utils2.default.extend(col, definedSchema);
          }
        }
        // set css name for column
        n = "css";
        if (!_utils2.default.isString(col[n])) {
          col[n] = _string2.default.kebabCase(col.name);
        }

        if (!_utils2.default.isString(col.displayName)) col.displayName = col.name;
        columns.push(col);
      }

      // if the user defined the columns inside the options;
      // automatically set their position on the basis of their index
      var p = "position";
      if (optionsColumns) {
        var i = 0;
        for (var x in optionsColumns) {
          var col = _utils2.default.find(columns, function (o) {
            return o.name == x;
          });
          if (col && !_utils2.default.has(col, p)) col[p] = i;
          i++;
        }
      }

      // restore columns information from cache
      //
      var columnsData = self.getCachedColumnsData();
      var h = "hidden";
      if (columnsData) {
        _utils2.default.each(columnsData, function (c) {
          var col = _utils2.default.find(columns, function (o) {
            return o.name == c.name;
          });
          if (col) {
            col[p] = c[p]; // position
            col[h] = c[h]; // hidden
          }
        });
      }
      self.columns = columns;
      // Now columns information is initialized:
      // it may be necessary to set a search filter, for fixed tables
      if (self.fixed && self.searchText) {
        self.setSearchFilter(self.searchText, true);
      }
      return self;
    }
  }, {
    key: "storePreference",
    value: function storePreference(plainKey, value) {
      var store = this.getFiltersStore();
      if (!store) return false;
      var key = this.getMemoryKey(plainKey);
      store.setItem(key, value);
    }
  }, {
    key: "getPreference",
    value: function getPreference(plainKey) {
      var store = this.getFiltersStore();
      if (!store) return;
      var key = this.getMemoryKey(plainKey);
      return store.getItem(key);
    }

    /**
     * Returns the storage used to store filters settings.
     */

  }, {
    key: "getFiltersStore",
    value: function getFiltersStore() {
      return localStorage;
    }

    /**
     * Returns the storage used to store data.
     */

  }, {
    key: "getDataStore",
    value: function getDataStore() {
      return sessionStorage;
    }

    /**
     * Loads settings from configured storage.
     */

  }, {
    key: "loadSettings",
    value: function loadSettings() {
      if (this.getTableData) {
        // table requires data: try to restore from cache
        this.restoreTableData();
      }
      return this.restoreFilters();
    }

    /**
     * Restore filters from cache.
     */

  }, {
    key: "restoreFilters",
    value: function restoreFilters() {
      var self = this,
          options = self.options;

      // restore filters from storage
      var filtersStore = self.getFiltersStore();
      if (!filtersStore) return self;

      var key = self.getMemoryKey("filters");
      var filtersCache = filtersStore.getItem(key);
      if (!filtersCache) return self;

      try {
        var filters = _json2.default.parse(filtersCache);
        var basicFilters = "page size sortBy search timestamp".split(" ");
        self.trigger("restore:filters", filters);
        // restore table inner filters
        _utils2.default.each(basicFilters, function (x) {
          // size maps to results per page! (because table 'size' would be unclear)
          if (x == "search") {
            if (self.validateForSeach(filters[x])) {
              self.setSearchFilter(filters[x], true);
            }
          } else if (x == "sortBy") {
            self.sortCriteria = filters[x];
          } else if (x == "size") {
            options.resultsPerPage = filters[x];
          } else {
            options[x] = filters[x];
          }
        });
        // call a function and fire an event, so the user of the library can restore
        // custom, table specific filters.
        var extraFilters = _utils2.default.minus(filters, basicFilters);
        if (!_utils2.default.isEmpty(extraFilters)) {
          self.restoreExtraFilters(filters);
        }
      } catch (ex) {
        // deserialization failed: remove item from cache
        filtersStore.removeItem(key);
      }
      return self;
    }

    /**
     * Returns the current filters of this KingTable.
     * Filters include page number, page size, order by (property name); sort criteria,
     * free text search, timestamp of the first time the table was rendered (this timestamp is updated when the user clicks on the refresh button).
     */

  }, {
    key: "getFilters",
    value: function getFilters() {
      var self = this,
          pagination = self.pagination;
      // NB: this function disallow overriding basic filters (page number, size, sortBy, search, timestamp);
      // using the following order of elements:
      var anchorTime = "anchorTime";
      if (_utils2.default.isUnd(self[anchorTime])) {
        // set anchor time: it can be used for fast growing tables.
        // this must be fetched before caching filters
        self[anchorTime] = new Date();
      }
      return _utils2.default.extend({}, self.getExtraFilters(), {
        page: pagination.page, // page number
        size: pagination.resultsPerPage, // page size; i.e. results per page
        sortBy: self.sortCriteria || null, // sort criteria (one or more properties)
        search: self.searchText || null,
        timestamp: self.anchorTime || null // the timestamp of the first time the table was rendered
      });
    }

    /**
     * Returns the current filters of this KingTable, including a caching mechanism (set only).
     */

  }, {
    key: "getFiltersSetCache",
    value: function getFiltersSetCache() {
      var self = this,
          filters = self.getFilters(),
          store = self.getFiltersStore();
      if (store) {
        // store current filters set; this is used to restore
        // filters upon reload or page refresh
        var key = self.getMemoryKey("filters");
        self.trigger("store:filters", filters);
        store.setItem(key, _json2.default.compose(filters));
      }
      return filters;
    }

    /**
     * Function that allows to return extra filters for a specific instance of KingtTable.
     * This function is used to collect filters that relates to a specific context, and merge them with
     * table basic filters (page number, page size, sort order).
     * Extensibility point.
     *
     * @return {object}
     */

  }, {
    key: "getExtraFilters",
    value: function getExtraFilters() {}

    /**
     * Function that allows to restore extra filters from cache, for a specific instance of KingtTable.
     * This function is used restore cached filters in the table context, so they can be read when fetching a collection of items.
     * Extensibility point.
     *
     * @return {object}
     */

  }, {
    key: "restoreExtraFilters",
    value: function restoreExtraFilters(filters) {}

    /**
     * Function to run before rendering.
     * Extensibility point.
     */

  }, {
    key: "beforeRender",
    value: function beforeRender() {}

    /**
     * Function to run after rendering.
     * Extensibility point.
     */

  }, {
    key: "afterRender",
    value: function afterRender() {}

    /**
     * Function to run when an AJAX request starts.
     * Extensibility point.
     */

  }, {
    key: "onFetchStart",
    value: function onFetchStart() {}

    /**
     * Function to run when an AJAX request ends positively.
     * Extensibility point.
     */

  }, {
    key: "onFetchDone",
    value: function onFetchDone() {}

    /**
     * Function to run when an AJAX request ends negatively.
     * Extensibility point.
     */

  }, {
    key: "onFetchFail",
    value: function onFetchFail() {}

    /**
     * Function to run when an AJAX request ends (in any case).
     * Extensibility point.
     */

  }, {
    key: "onFetchEnd",
    value: function onFetchEnd() {}

    /**
     * Function to run when search filter is empty.
     * Extensibility point.
     */

  }, {
    key: "onSearchEmpty",
    value: function onSearchEmpty() {}

    /**
     * Function to run when search starts.
     * Extensibility point.
     */

  }, {
    key: "onSearchStart",
    value: function onSearchStart(val) {}

    /**
     * Define a function that allows to preprocess data upon fetching.
     * Extensibility point.
     */

  }, {
    key: "prepareData",
    value: function prepareData(data) {
      // handle this.data (e.g. parsing dates, etc.)
      // data === this.data
      return this;
    }

    /**
     * Formats the values inside the items (requires columns information).
     */

  }, {
    key: "formatValues",
    value: function formatValues(data) {
      // first use the function that is designed to be overridable by programmers
      var self = this,
          o = self.options;
      if (!data) data = self.data;
      // apply formatting by column
      var formattedSuffix = self.options.formattedSuffix,
          n,
          v;
      var formattedProperties = _utils2.default.where(self.columns, function (x) {
        return _utils2.default.isFunction(x.format);
      });
      _utils2.default.each(data, function (x) {
        _utils2.default.each(formattedProperties, function (c) {
          n = c.name + formattedSuffix;
          v = x[c.name];
          x[n] = _utils2.default.isUnd(v) || v === null || v === "" ? o.emptyValue : c.format(v, x) || o.emptyValue;
        });
      });
      return self;
    }

    /**
     * Sets the columns order by property name.
     */

  }, {
    key: "setColumnsOrder",
    value: function setColumnsOrder() {
      var args = _utils2.default.stringArgs(arguments);
      var l = args.length;
      if (!l) return false;
      var cols = this.columns,
          found = [];
      for (var i = 0; i < l; i++) {
        var n = args[i];
        var col = _utils2.default.find(cols, function (x) {
          return x.name == n;
        });
        if (!col) (0, _raise2.default)(19, "missing column with name \"" + n + "\"");
        col.position = i;
        found.push(col);
      }
      var notFound = _utils2.default.where(cols, function (x) {
        return found.indexOf(x) == -1;
      });
      _utils2.default.each(notFound, function (x) {
        l++;
        x.position = l;
      });
      _array2.default.sortBy(cols, "position");
      // store in memory
      this.storeColumnsData().render();
      return this;
    }

    /**
     * Shows or hides columns, depending on parameter.
     */

  }, {
    key: "toggleColumns",
    value: function toggleColumns(param) {
      var cols = this.columns;
      _utils2.default.each(param, function (x) {
        if (_utils2.default.isArray(x)) {
          var name = x[0],
              visible = x[1];
        } else {
          var name = x.name,
              visible = x.visible;
        }

        var col = _utils2.default.find(cols, function (col) {
          return col.name == name;
        });
        if (!col) {
          (0, _raise2.default)(19, "missing column with name \"" + name + "\"");
        }
        if (visible) {
          col.hidden = col.secret ? true : false;
        } else {
          col.hidden = true;
        }
      });
      this.storeColumnsData().render();
      return this;
    }

    /**
     * Hides one or more columns by name.
     */

  }, {
    key: "hideColumns",
    value: function hideColumns() {
      return this.columnsVisibility(_utils2.default.stringArgs(arguments), false);
    }

    /**
     * Shows one or more columns by name.
     */

  }, {
    key: "showColumns",
    value: function showColumns() {
      return this.columnsVisibility(_utils2.default.stringArgs(arguments), true);
    }

    /**
     * Sets columns visibility.
     */

  }, {
    key: "columnsVisibility",
    value: function columnsVisibility(args, visible) {
      var _this2 = this;

      if (args.length == 1 && args[0] == "*") {
        _utils2.default.each(this.columns, function (x) {
          if (visible) {
            x.hidden = x.secret ? true : false;
          } else {
            x.hidden = true;
          }
        });
      } else {
        _utils2.default.each(args, function (x) {
          _this2.colAttr(x, "hidden", !visible);
        });
      }
      this.storeColumnsData().render();
      return this;
    }

    /**
     * Sets a column property by names and value.
     *
     * @param {string} name: column name
     * @param {string} attr: attribute name
     * @param {any} value: attribute value
     */

  }, {
    key: "colAttr",
    value: function colAttr(name, attr, value) {
      if (!name) (0, _exceptions.ArgumentNullException)("name");
      var cols = this.columns;
      if (!cols) (0, _raise2.default)(20, "missing columns information (properties not initialized)");
      var col = _utils2.default.find(cols, function (x) {
        return x.name == name;
      });
      if (!col) (0, _raise2.default)(19, "missing column with name \"" + name + "\"");
      col[attr] = value;
      return col;
    }

    /**
     * Stores columns data in cache.
     * NB: this is used only to store columns position, hidden data.
     * Objects structure still controls the columns data.
     */

  }, {
    key: "storeColumnsData",
    value: function storeColumnsData() {
      var store = this.getDataStore(),
          key = this.getMemoryKey("columns:data"),
          options = this.options,
          data = this.columns;
      if (store && options.storeTableData) {
        store.setItem(key, _json2.default.compose(data));
      }
      return this;
    }

    /**
     * Gets columns data from storage.
     * NB: this is used only to store columns position, hidden data.
     * Objects structure still controls the columns data.
     */

  }, {
    key: "getCachedColumnsData",
    value: function getCachedColumnsData() {
      var store = this.getDataStore(),
          key = this.getMemoryKey("columns:data"),
          options = this.options;
      if (store && options.storeTableData) {
        var data = store.getItem(key);
        if (data) {
          try {
            return _json2.default.parse(data);
          } catch (ex) {
            store.removeItem(key);
          }
        }
      }
      return null;
    }

    /**
     * Sorts columns, according to their current position setting.
     */

  }, {
    key: "sortColumns",
    value: function sortColumns() {
      if (arguments.length) {
        return this.setColumnsOrder.apply(this, arguments);
      }
      // default function to sort columns: they are sorted
      // by position first, then display name
      var isNumber = _utils2.default.isNumber,
          columns = this.columns;
      columns.sort(function (a, b) {
        var p = "position";
        if (isNumber(a[p]) && !isNumber(b[p])) return -1;
        if (!isNumber(a[p]) && isNumber(b[p])) return 1;
        if (a[p] > b[p]) return 1;
        if (a[p] < b[p]) return -1;
        // compare display name
        p = "displayName";
        return _string2.default.compare(a[p], b[p], 1);
      });
      for (var i = 0, l = columns.length; i < l; i++) {
        columns[i].position = i;
      }return this;
    }
  }, {
    key: "setTools",
    value: function setTools() {
      // TODO: keep this class abstracted from DOM
      return this;
    }

    /**
     * Allows to define a function that returns data required to render the table itself.
     * This is commonly necessary, for example, when an AJAX request is required to fetch filters information
     * (e.g. an array of possible types for a select)
     * Extensibility point.
     */
    // getTableData() { }

    /**
     * Handles data returned by the 'getTableData' promise, if any.
     * By default, this function simply stores the table data in a property called 'tableData'
     * Extensibility point.
     *
     * @param {object} data: data returned by getTableData promise (if any).
     */

  }, {
    key: "handleTableData",
    value: function handleTableData(data) {
      this.tableData = data;
    }

    /*
     * Refreshes the KingTable, clearing its data cache and
     * performing a new rendering.
     */

  }, {
    key: "refresh",
    value: function refresh() {
      delete this.anchorTime;
      // clear data cache

      return this.render({
        clearDataCache: true
      });
    }

    /**
     * Performs an hard refresh on the KingTable; clearing all its cached data
     * and performing a new rendering.
     */

  }, {
    key: "hardRefresh",
    value: function hardRefresh() {
      this.trigger("hard:refresh").clearTableData();
      return this.render({
        clearDataCache: true
      });
    }

    /**
     * Renders the KingTable, using its current view and view builder.
     * If necessary, it also fetches data required by the table itself
     * (e.g. information to render the filters view).
     */

  }, {
    key: "render",
    value: function render(options) {
      var self = this;

      return new Promise(function (resolve, reject) {
        function handle() {
          self.beforeRender();

          self.initColumns().sortColumns().setTools().build();
          self.afterRender();
          resolve();
        }

        function callback() {
          // TODO: the `hasData` check is not specific enough. (maybe?)
          if (self.fixed && self.hasData()) {
            //
            // resolve automatically: the data is already available and not changing
            // however, filters need to be stored for consistency with paginated sets.
            self.getFiltersSetCache();
            handle();
          } else {
            // it is necessary to fetch data
            var timestamp = self.lastFetchTimestamp = new Date().getTime();
            self.getList(options, timestamp).then(function success(data) {
              if (!data || !data.length && !self.columnsInitialized) {
                // there is no data: this may happen if the server is not returning any
                // object
                return self.emit("no-results");
              }
              handle();
            }, function fail() {
              self.emit("get-list:failed");
              reject("get-list:failed");
            });
          }
        }

        //
        // if necessary, fetch list data (e.g. filters data, or anything else that need to be fetched
        // from the server side before displaying a rendered table)
        //
        if (self.getTableData && !self.cache.tableDataFetched) {
          // was the data already fetched?

          var tableDataPromise = self.getTableData();
          // NB: here duck typing is used.
          if (!_utils2.default.quacksLikePromise(tableDataPromise)) {
            (0, _raise2.default)(13, "getTableData must return a Promise or Promise-like object.");
          }

          tableDataPromise.then(function (data) {
            if (self.options.storeTableData) {
              self.cache.tableDataFetched = true;
              // store table data in store
              self.storeTableData(data);
            }
            self.handleTableData(data);
            callback();
          }, function () {
            // fetching table data failed
            self.emit("get-table-data:failed");
            reject();
          });
        } else {
          // table requires no data
          callback();
        }
      });
    }

    /**
     * Allows to clear stored table data.
     */

  }, {
    key: "clearTableData",
    value: function clearTableData() {
      var self = this,
          store = self.getDataStore(),
          key = self.getMemoryKey("table:data"),
          options = self.options;
      if (store && options.storeTableData) {
        store.removeItem(key);
      }
      self.cache.tableDataFetched = false;
      delete self.tableData;
      return self;
    }

    /**
     * Stores table data for later use.
     */

  }, {
    key: "storeTableData",
    value: function storeTableData(data) {
      var store = this.getDataStore(),
          key = this.getMemoryKey("table:data"),
          options = this.options;
      if (store && options.storeTableData) {
        store.setItem(key, _json2.default.compose(data));
      }
      return this;
    }

    /**
     * Restores table data from cache.
     */

  }, {
    key: "restoreTableData",
    value: function restoreTableData() {
      var self = this,
          store = self.getDataStore(),
          key = self.getMemoryKey("table:data"),
          options = self.options;
      if (store && options.storeTableData) {
        var data = store.getItem(key);
        if (data) {
          // table data is available in cache
          try {
            // TODO: use the same function to parse, used by ajax proxy functions
            data = _json2.default.parse(data);
          } catch (ex) {
            // parsing failed
            store.removeItem(key);
          }
          self.handleTableData(data);
          self.cache.tableDataFetched = true;
        }
      }
      return self;
    }

    /**
     * Builds the KingTable, using its current state and view builder.
     */

  }, {
    key: "build",
    value: function build() {
      // depending on the current view, use the right builder to build
      // the table at its current state.
      var self = this;
      var builder = self.builder;
      if (!builder) {
        return;
      }
      builder.build();
    }

    /**
     * Returns a memory key for this KingTable.
     */

  }, {
    key: "getMemoryKey",
    value: function getMemoryKey(name) {
      var a = location.pathname + location.hash + ".kt";
      var id = this.id;
      if (id) a = id + ":" + a;
      return name ? a + ":::" + name : a;
    }

    /**
     * Handles fixed data.
     */

  }, {
    key: "setFixedData",
    value: function setFixedData(data) {
      var self = this;
      data = self.normalizeCollection(data);
      self.fixed = true;
      self.filters.searchDisabled = false;
      self.prepareData(data);
      self.data = data;
      self.initColumns();
      self.formatValues(data);
      self.updatePagination(data.length);
      return data;
    }

    /**
     * Fetches the data for this KingTable and handles its response.
     * The default implementation performs an ajax request.
     */

  }, {
    key: "getList",
    value: function getList(options, timestamp) {
      options = options || {};
      var self = this;
      // obtain fetch options
      var fetchOptions = self.mixinFetchData();

      return new Promise(function (resolve, reject) {
        self.emit("fetch:start").onFetchStart();
        self.getFetchPromiseWithCache(fetchOptions, options).then(function done(data) {
          // check if there is a newer call to function
          if (timestamp < self.lastFetchTimestamp) {
            // do nothing because there is a newer call to loadData
            return;
          }
          if (!data) {
            // invalid promise: the function must return something when resolving
            (0, _raise2.default)(14, "`getFetchPromise` did not return a value when resolving");
          }
          self.emit("fetch:done").onFetchDone(data);

          // check if returned data is an array or a catalog
          if (_utils2.default.isArray(data)) {
            //
            // The server returned an array, so take for good that this collection
            // is complete and doesn't require server side pagination. This is by design.
            //
            data = self.setFixedData(data);
            //
            // The collection is complete: apply client side pagination
            //
            resolve(self.getSubSet(data));
          } else {
            //
            // The server returned an object, so take for good that this collection requires
            // server side pagination; expect the returned data to include information like:
            // total number of results (possibly), so a client side pagination can be built;
            //
            // expect catalog structure (page count, page number, etc.)
            var subset = data.items || data.subset;
            if (!_utils2.default.isArray(subset)) (0, _raise2.default)(6, "The returned object is not a catalog");
            if (!_utils2.default.isNumber(data.total)) (0, _raise2.default)(7, "Missing total items count in response object.");

            subset = self.normalizeCollection(subset);
            // set data
            self.prepareData(subset);
            self.data = subset;
            self.initColumns();
            self.formatValues(subset);
            self.updatePagination(data.total);
            resolve(subset);
          }
        }, function fail() {
          // check if there is a newer call to function
          if (timestamp < self.lastFetchTimestamp) {
            // do nothing because there is a newer call to loadData
            return;
          }
          self.emit("fetch:fail").onFetchFail();
          reject();
        }).then(function always() {
          self.emit("fetch:end").onFetchEnd();
        });
      });
    }

    /**
     * Performs a search by text.
     */

  }, {
    key: "search",
    value: function search(val) {
      if (_utils2.default.isUnd(val)) val = "";
      var self = this;
      if (self.validateForSeach(val)) {
        // add filters inside the filters manager
        if (!val) {
          // remove filter
          self.unsetSearch();
        } else {
          self.onSearchStart(val);
          self.setSearchFilter(val);
        }
        // go to first page
        self.pagination.page = 1;
      } else {
        // value is not valid for search: remove the rule by key
        self.unsetSearch();
      }
      self.render();
    }
  }, {
    key: "isSearchActive",
    value: function isSearchActive() {
      var filter = this.filters.getRuleByKey("search");
      return !!filter;
    }

    /**
     * Unsets the search filters in this table.
     */

  }, {
    key: "unsetSearch",
    value: function unsetSearch() {
      var self = this;
      if (!self.isSearchActive()) {
        return self;
      }
      self.filters.removeRuleByKey("search");
      self.searchText = null;
      if (self.hasData()) self.updatePagination(self.data.length);
      self.trigger("search-empty").onSearchEmpty();
      return self;
    }

    /**
     * Sets a search filter for the table.
     *
     * @param {string} val: search value.
     * @param {bool} skipStore: whether to skip storing the filter in cache or not.
     */

  }, {
    key: "setSearchFilter",
    value: function setSearchFilter(val, skipStore) {
      var self = this;
      self.searchText = val;
      if (!skipStore) {
        self.getFiltersSetCache(); // store filter in cache
      }
      var searchProperties = self.getSearchProperties();
      // NB: if data is fetched from the server after table initialization,
      // then searchProperties may still be not available.
      self.filters.set({
        type: "search",
        key: "search",
        value: _regex2.default.getSearchPattern(_string2.default.getString(val), {
          searchMode: self.options.searchMode
        }),
        searchProperties: searchProperties && searchProperties.length ? searchProperties : false
      });
      self.trigger("search-active");
      return self;
    }

    /**
     * Gets the properties that should be used to search in this table.
     */

  }, {
    key: "getSearchProperties",
    value: function getSearchProperties() {
      var self = this,
          options = self.options;
      if (options.searchProperties)
        // the user explicitly specified the search properties
        return options.searchProperties;

      // if data is not initialized yet, return false; search properties will be set later
      if (!self.data || !self.columnsInitialized) return false;

      var searchable = _utils2.default.where(self.columns, function (col) {
        return col.allowSearch && !col.secret; // exclude secret columns
      });
      //
      // When searching, it's desirable to search inside string representations of
      // values, while keeping real values in the right type (for sorting numbers and dates, for instance)
      // However, it is also nice to search by actual values (e.g. searching "1000" should match numbers that are represented with thousands separators, too --> like '1,000.00')
      //
      var formattedSuffix = options.formattedSuffix;
      return _utils2.default.flatten(_utils2.default.map(searchable, function (x) {
        if (_utils2.default.isFunction(x.format)) {
          // TODO: which properties should also be searched in their default string representation? (Dates not likely, Numbers most probably yes)
          return x.type == "number" ? [x.name + formattedSuffix, x.name] : x.name + formattedSuffix;
        }
        return x.name;
      }));
    }

    /**
     * Returns true if a string value should trigger a search, false otherwise.
     */

  }, {
    key: "validateForSeach",
    value: function validateForSeach(val) {
      if (!val) return false;
      var minSearchChars = this.options.minSearchChars;
      if (val.match(/^[\s]+$/g) || _utils2.default.isNumber(minSearchChars) && val.length < minSearchChars) {
        return false;
      }
      return true;
    }

    /**
     * Returns a string representation of the anchor fetch time.
     * Time used to `anchor` fetching of items for fast growing tables
     * (i.e. items can be fetched if their creation time is before anchor timestamp)
     */

  }, {
    key: "getFormattedAnchorTime",
    value: function getFormattedAnchorTime() {
      var time = this.anchorTime;
      if (time instanceof Date) {
        if (_date2.default.isToday(time)) {
          return _date2.default.format(time, "HH:mm:ss");
        }
        return _date2.default.formatWithTime(time);
      }
      return "";
    }

    /**
     * Returns a string representation of the data fetch time.
     */

  }, {
    key: "getFormattedFetchTime",
    value: function getFormattedFetchTime() {
      var time = this.dataFetchTime;
      if (time instanceof Date) {
        if (_date2.default.isToday(time)) {
          return _date2.default.format(time, "HH:mm:ss");
        }
        return _date2.default.formatWithTime(time);
      }
      return "";
    }

    /**
     * Returns a promise object responsible of fetching data including a LRU caching mechanism;
     *
     * @param params
     * @returns {Promise}
     */

  }, {
    key: "getFetchPromiseWithCache",
    value: function getFetchPromiseWithCache(params, options) {
      // LRU caching mechanism. If the fetch options didn't change (it means: same filters),
      // and there is already data in the local storage or session storage, use stored data.
      if (!options) options = {};
      var self = this,
          o = self.options,
          lruCacheSize = o.lruCacheSize,
          store = self.getDataStore(),
          useLru = !!(lruCacheSize && store);
      var anchorTime = "anchorTime";
      if (useLru) {
        // check if there is data in the store
        var frozen = _json2.default.parse(_json2.default.compose(params));
        var key = self.getMemoryKey("catalogs"),
            cachedData = _lru2.default.get(key, function (x) {
          return _utils2.default.equal(frozen, x.filters);
        }, store, true);
        if (cachedData) {
          if (options.clearDataCache) {
            // clear the cache for all pages,
            // this is important to not confuse the user
            // because if only the cache for a specific page number were cleared, it would be difficult to understand
            _lru2.default.remove(key, undefined, store);
          } else {
            // set timestamp of when data was fetched
            self[anchorTime] = new Date(cachedData.data[anchorTime]);
            self.dataFetchTime = new Date(cachedData.ts);
            return new Promise(function (resolve, reject) {
              //
              // NB: it is important to use a timeout of 0 milliseconds, to
              // recreate similar scenario like the one given by an AJAX request
              // (e.g. for libraries like Knockout or Vue.js)
              // Otherwise the view would be build in different ways
              //
              setTimeout(function () {
                resolve(cachedData.data.data);
              }, 0);
            });
          }
        }
      }
      // fetch remotely
      return new Promise(function (resolve, reject) {
        self.loading = true;
        self.emit("fetching:data");
        self.getFetchPromise(params).then(function done(data) {
          if (useLru) {
            // store in cache
            _lru2.default.set(key, {
              data: data,
              filters: params,
              anchorTime: self[anchorTime].getTime()
            }, o.lruCacheSize, o.lruCacheMaxAge, store);
          }
          self.dataFetchTime = new Date();
          self.loading = false;
          self.emit("fetched:data");
          resolve(data);
        }, function fail() {
          self.loading = false;
          reject();
        });
      });
    }

    /**
     * Returns a promise object responsible of fetching data;
     * Override this function if data should be fetched in other ways
     * (for example, reading a file from file system).
     * This function must return a Promise object or a compatible object.
     *
     * @param params
     * @returns {Promise}
     */

  }, {
    key: "getFetchPromise",
    value: function getFetchPromise(params) {
      // The default implementation implements getFetchPromise by generating an AJAX call,
      // since this is the most common use case scenario.
      // However, this class is designed to be almost abstracted from AJAX, so overriding this single
      // function allows to fetch data from other sources (e.g. reading files in chunks from file system; returning mock data for unit tests; etc).
      var options = this.options;
      var url = options.url;
      if (!url) (0, _raise2.default)(5, "Missing url option, to fetch data");

      // NB: if method is GET, ajax helper will automatically convert it to a query string
      // with keys in alphabetical order; conversion is done transparently
      var method = options.httpMethod;

      // format fetch data
      params = this.formatFetchData(params);

      return _ajax2.default.shot({
        type: method,
        url: url,
        data: params
      });
    }
  }, {
    key: "numberFilterFormatter",
    value: function numberFilterFormatter(propertyName, value) {
      return value;
    }
  }, {
    key: "dateFilterFormatter",
    value: function dateFilterFormatter(propertyName, value) {
      return _date2.default.toIso8601(value);
    }
  }, {
    key: "formatFetchData",
    value: function formatFetchData(data) {
      var options = this.options;
      var sortByFormatter = options.sortByFormatter;
      if (data.sortBy && _utils2.default.isFunction(sortByFormatter)) {
        data.sortBy = sortByFormatter(data.sortBy);
      }

      var x;
      for (x in data) {
        var v = data[x];
        if (v instanceof Date) {
          data[x] = this.dateFilterFormatter(x, v);
        }
        if (v instanceof Number) {
          data[x] = this.numberFilterFormatter(x, v);
        }
      }
      return data;
    }

    /**
     * Returns an object that describe all filters and necessary options to fetch data.
     */

  }, {
    key: "mixinFetchData",
    value: function mixinFetchData() {
      var extraData = this.options.fetchData;
      if (_utils2.default.isFunction(extraData)) extraData = extraData.call(this);
      return _utils2.default.extend(this.getFiltersSetCache(), extraData || {});
    }

    /**
     * Ensures that a collection is normalized, if the server is returning an optimized
     * collection in the shape of array of arrays.
     * Optimized collections are converted in dictionaries, for easier handling during rendering,
     * as values can be handled by property name instead of array index.
     *
     * @param collection: collection to normalize
     */

  }, {
    key: "normalizeCollection",
    value: function normalizeCollection(collection) {
      var l = collection.length;
      if (!l) return collection;
      var first = collection[0];
      if (_utils2.default.isArray(first)) {
        // assumes that the server is returning an optimized collection:
        // the first array contains the column names; while the others the values.
        var a = [],
            i,
            j = first.length,
            k;
        for (i = 1; i < l; i++) {
          var o = {};
          for (k = 0; k < j; k++) {
            o[first[k]] = collection[i][k];
          }
          a.push(o);
        }
        return a;
      }
      // the collection is not optimized
      return collection;
    }

    /**
     * Returns the current collection of items.
     */

  }, {
    key: "getData",
    value: function getData(options) {
      var o = _utils2.default.extend({
        optimize: false,
        itemCount: true,
        hide: true // whether to exclude `hidden` columns
      }, options),
          self = this,
          itemCount = self.options.itemCount && o.itemCount;
      var data = self.getItemsToDisplay();
      var columns = _utils2.default.clone(self.columns);
      if (o.hide) {
        // delete hidden properties
        _utils2.default.each(_utils2.default.where(self.columns, function (o) {
          return o.hidden || o.secret;
        }), function (o) {
          _utils2.default.each(data, function (d) {
            delete d[o.name];
          });
        });
        columns = _utils2.default.where(self.columns, function (o) {
          return !o.hidden && !o.secret;
        });
      }
      if (itemCount) {
        self.setItemsNumber(data);
      }
      if (o.optimize) {
        if (itemCount) {
          columns.unshift({
            name: "ε_row",
            displayName: "#"
          });
        }
        return self.optimizeCollection(data, columns, o);
      }
      return data;
    }

    /**
     * Returns a list of items to display for this table at its current state.
     */

  }, {
    key: "getItemsToDisplay",
    value: function getItemsToDisplay() {
      var self = this,
          options = self.options,
          data = self.data;
      if (!data || !data.length) return [];
      //
      // clone the data; this is required to alter items
      // without affecting original items
      //
      data = _utils2.default.clone(data);

      if (!self.fixed) return data;

      // paginate, filter and sort client side
      var l = data.length;
      // apply filters here
      data = self.filters.skim(data);
      if (data.length != l) {
        self.updatePagination(data.length);
      }
      // apply sorting logic, but only if there is no search specified (the search by string property is already sorting really well),
      // or if the search sorting is configured to not rules over the regular sorting
      if (!self.searchText || !options.searchSortingRules) {
        var sortCriteria = self.sortCriteria;
        if (!_utils2.default.isEmpty(sortCriteria)) {
          _array2.default.sortBy(data, sortCriteria);
        }
      }
      // obtain a subset of data, after sorting (this order is really important)
      var subset = self.getSubSet(data);
      return subset;
    }

    /**
     * Sorts the underlying items by one or more properties.
     *
     * @param {(string|string[]|objects)} criteria: object describing
     */

  }, {
    key: "sortBy",
    value: function sortBy() {
      var criteria = _array2.default.getSortCriteria(arguments);
      if (!criteria || !criteria.length) {
        return this.unsetSortBy();
      }
      var self = this;
      self.sortCriteria = criteria;
      if (self.hasData()) {
        // render (will trigger storing filters in cache)
        _array2.default.sortBy(self.data, criteria);
        self.render();
      } else {
        // store filters in cache
        self.getFiltersSetCache();
      }
      return self;
    }

    /**
     * Progress sort order for a property with the given name.
     */

  }, {
    key: "progressSortBy",
    value: function progressSortBy(name) {
      if (!name) (0, _exceptions.ArgumentNullException)("name");
      var self = this;
      var columns = self.columns;
      if (!columns) {
        // this function can be called only when columns information are initialized
        (0, _raise2.default)(20, "Missing columns information");
      }
      var property = _utils2.default.find(columns, function (x) {
        return x.name == name;
      });
      if (!property) {
        (0, _raise2.default)(19, "Column '${name}' is not found among columns.");
      }
      var criteria = self.sortCriteria || [];
      var existingSort = _utils2.default.find(criteria, function (x) {
        return x[0] == name;
      });
      if (!existingSort) {
        // start by ascending, by default
        criteria.push([name, 1]);
      } else {
        var order = existingSort[1];
        if (order === -1) {
          // remove
          criteria = _utils2.default.reject(criteria, function (x) {
            return x[0] == name;
          });
        } else {
          // order can only be 1, move by 1
          existingSort[1] = -1;
        }
      }
      self.sortBy(criteria);
    }

    /**
     * Progress sort order for a single property with the given name.
     */

  }, {
    key: "progressSortBySingle",
    value: function progressSortBySingle(name) {
      if (!name) (0, _exceptions.ArgumentNullException)("name");
      var self = this;
      var columns = self.columns;
      if (!columns) {
        // this function can be called only when columns information are initialized
        (0, _raise2.default)(20, "Missing columns information");
      }
      var property = _utils2.default.find(columns, function (x) {
        return x.name == name;
      });
      if (!property) {
        (0, _raise2.default)(19, "Column '${name}' is not found among columns.");
      }
      var criteria = self.sortCriteria || [];
      var existingSort = _utils2.default.find(criteria, function (x) {
        return x[0] == name;
      });
      if (!existingSort) {
        // start by ascending, by default
        criteria = [name, 1];
      } else {
        var order = existingSort[1];
        if (order === -1) {
          criteria = [name, 1];
        } else {
          criteria = [name, -1];
        }
      }
      self.sortBy([criteria]);
    }

    /**
     * Unsets the sort by criteria for this table.
     */

  }, {
    key: "unsetSortBy",
    value: function unsetSortBy() {
      this.sortCriteria = null;
      // render (will trigger storing filters in cache)
      this.render();
      return this;
    }

    /**
     * Sets the numeration inside a given array of items.
     */

  }, {
    key: "setItemsNumber",
    value: function setItemsNumber(arr) {
      var self = this,
          pag = self.pagination,
          offset = (pag.page - 1) * pag.resultsPerPage;
      if (!arr) arr = self.data;
      var l = arr.length;
      for (var i = 0; i < l; i++) {
        arr[i].ε_row = (i + 1 + offset).toString();
      }
      return arr;
    }

    /**
     * Optimizes a collection; making its structure smaller by removing the property names.
     *
     * @param data
     * @param {string[]} columns: columns to include.
     * @param {object} options: options to optimize the collection.
     */

  }, {
    key: "optimizeCollection",
    value: function optimizeCollection(data, columns, options) {
      if (!columns) columns = this.columns;
      if (!options) options = {
        format: true
      };
      var a = [_utils2.default.map(columns, function (o) {
        return o.displayName;
      })],
          len = "length",
          push = "push",
          format = options.format,
          formattedSuffix = this.options.formattedSuffix,
          obj;
      for (var i = 0, l = data[len]; i < l; i++) {
        var b = [];
        for (var k = 0, j = columns[len]; k < j; k++) {
          var colname = columns[k].name,
              formattedName = colname + formattedSuffix;
          obj = data[i];
          if (format && _utils2.default.has(obj, formattedName)) {
            b[push](obj[formattedName]);
          } else {
            // NB: if the object does not have a property, string empty is added
            // to fill the property place.
            b[push](obj[colname] || "");
          }
        }
        a[push](b);
      }
      return a;
    }

    /**
     * Returns the value of the given property from the given item,
     * eventually returning the formatted value.
     */

  }, {
    key: "getItemValue",
    value: function getItemValue(item, name) {
      if (!item) (0, _exceptions.ArgumentNullException)("item");
      var options = this.options,
          formattedSuffix = options.formattedSuffix,
          formattedName = name + formattedSuffix;
      return _utils2.default.has(item, formattedName) ? item[formattedName] : item[name];
    }

    /**
     * Default function to get the name of the id property of displayed objects.
     */

  }, {
    key: "getIdProperty",
    value: function getIdProperty() {
      var o = this.options;
      if (_utils2.default.isString(o.idProperty)) return o.idProperty;

      var columns = this.columns;
      if (!columns || !columns.length) (0, _raise2.default)(4, "id property cannot be determined: columns are not initialized.");

      for (var i = 0, l = columns.length; i < l; i++) {
        var name = columns[i].name;
        if (/^_?id$|^_?guid$/i.test(name)) return name;
      }
      (0, _raise2.default)(4, "id property cannot be determined, please specify it using 'idProperty' option.");
    }
  }, {
    key: "getExportFileName",
    value: function getExportFileName(format) {
      return this.options.collectionName + "." + format;
    }
  }, {
    key: "getColumnsForExport",
    value: function getColumnsForExport() {
      var columns = this.columns;
      return this.options.exportHiddenProperties ? columns : _utils2.default.reject(columns, function (o) {
        return o.hidden || o.secret;
      });
    }

    /**
     * Client side export for a specific format.
     */

  }, {
    key: "exportTo",
    value: function exportTo(format) {
      if (!format) (0, _exceptions.ArgumentException)("format");

      var self = this,
          options = self.options;
      var filename = self.getExportFileName(format),
          exportFormat = _utils2.default.find(self.options.exportFormats, function (o) {
        return o.format === format;
      }),
          columns = self.getColumnsForExport();
      if (!exportFormat || !exportFormat.type) (0, _raise2.default)(30, "Missing format information");

      var itemsToDisplay = self.getData({ itemCount: false });
      var contents = "";
      if (exportFormat.handler) {
        //user defined handler
        contents = exportFormat.handler.call(self, itemsToDisplay);
      } else {
        //use default export handlers
        switch (format) {
          case "csv":
            var data = self.optimizeCollection(itemsToDisplay);
            contents = _csv2.default.serialize(data, options.csvOptions);
            break;
          case "json":
            contents = _json2.default.compose(itemsToDisplay, 2, 2);
            break;
          case "xml":
            contents = self.dataToXml(itemsToDisplay);
            break;
          default:
            throw "export format " + format + "not implemented";
        }
      }
      if (contents) _file2.default.exportfile(filename, contents, exportFormat.type);
    }

    /**
     * Basic function to convert the given data into an xml structure.
     */

  }, {
    key: "dataToXml",
    value: function dataToXml(data) {
      var self = this,
          columns = self.getColumnsForExport(),
          options = self.options,
          len = "length",
          d = document,
          s = new XMLSerializer(),
          createElement = "createElement",
          appendChild = "appendChild",
          root = d[createElement](options.collectionName || "collection");

      for (var i = 0, l = data[len]; i < l; i++) {
        var item = d[createElement](options.entityName || "item");
        for (var k = 0, j = columns[len]; k < j; k++) {
          var col = columns[k],
              name = col.name,
              value = data[i][name];
          if (options.entityUseProperties) {
            //use properties
            item.setAttribute(name, value);
          } else {
            //use elements
            var subitem = d[createElement](name);
            subitem.innerText = value;
            item[appendChild](subitem);
          }
        }
        root[appendChild](item);
      }
      var a = s.serializeToString(root);
      return options.prettyXml ? _xml2.default.pretty(a) : _xml2.default.normal(a);
    }

    /**
     * 
     * @param {*} obj 
     */

  }, {
    key: "disposeOf",
    value: function disposeOf(obj) {
      obj.dispose();
      _utils2.default.removeItem(this.disposables, obj);
    }

    /**
     * Disposes this KingTable.
     */

  }, {
    key: "dispose",
    value: function dispose() {
      delete this.context;
      delete this.search;
      delete this.filters.context;
      _utils2.default.each(this.disposables, function (x) {
        if (x.dispose) x.dispose();
        if (_utils2.default.isFunction(x)) x();
      });
      this.disposables = [];
      var o = this.options;
      _utils2.default.ifcall(o.onDispose, this);
    }
  }], [{
    key: "regional",
    get: function get() {
      return _kingtable2.default;
    }
  }, {
    key: "version",
    get: function get() {
      return VERSION;
    }
  }, {
    key: "Utils",
    get: function get() {
      return _utils2.default;
    }
  }, {
    key: "StringUtils",
    get: function get() {
      return _string2.default;
    }
  }, {
    key: "NumberUtils",
    get: function get() {
      return _number2.default;
    }
  }, {
    key: "ArrayUtils",
    get: function get() {
      return _array2.default;
    }
  }, {
    key: "DateUtils",
    get: function get() {
      return _date2.default;
    }
  }, {
    key: "json",
    get: function get() {
      return _json2.default;
    }
  }, {
    key: "Paginator",
    get: function get() {
      return _paginator2.default;
    }
  }, {
    key: "PlainTextBuilder",
    get: function get() {
      return _kingtableText2.default;
    }
  }, {
    key: "HtmlBuilder",
    get: function get() {
      return _kingtableHtml2.default;
    }
  }, {
    key: "RichHtmlBuilder",
    get: function get() {
      return _kingtableRhtml2.default;
    }
  }, {
    key: "BaseHtmlBuilder",
    get: function get() {
      return _kingtableHtmlBase2.default;
    }

    /**
     * Gives access to KingTable builders, to allow overriding their functions.
     */

  }, {
    key: "builders",
    get: function get() {
      return BUILDERS;
    }

    /**
     * Gives access to KingTable custom storages, implementing an interface compatible
     * with sessionStorage and localStorage.
     */

  }, {
    key: "stores",
    get: function get() {
      return {
        "memory": _memstore2.default
      };
    }
  }]);

  return KingTable;
}(_events2.default);

// Extend KingTable object with properties that are meant to be globally available and editable
// for users of the library (programmers)
// NB: static get wouldn't work because the object would not be editable.
//


KingTable.defaults = DEFAULTS;

KingTable.Schemas = {
  /**
   * Default columns properties, by field value type.
   * This object is meant to be extended by implementers; following their personal preferences.
   */
  DefaultByType: {
    number: function number(columnSchema, objSchema) {
      return {
        format: function format(value) {
          // NB: this function is used only if a formatter function is not
          // defined for the given property; so here we suggest a format that makes sense for the value.
          return _number2.default.format(value);
        }
      };
    },
    date: function date(columnSchema, objSchema) {
      return {
        format: function dateFormatter(value) {
          // NB: this function is used only if a formatter function is not
          // defined for the given property; so here we suggest a format that makes sense for the value.

          // support date format defined inside column schema
          // use a format that makes sense for the value
          // if the date has time component, use format that contains time; otherwise only date part
          var hasTime = KingTable.DateUtils.hasTime(value);
          var format = KingTable.DateUtils.defaults.format[hasTime ? "long" : "short"];
          return KingTable.DateUtils.format(value, format);
        }
      };
    }
  },

  /**
   * Default columns properties, by field name.
   * This object is meant to be extended by implementers; following their personal preferences.
   */
  DefaultByName: {
    id: {
      name: "id",
      type: "id",
      hidden: true,
      secret: true
    },
    guid: {
      name: "guid",
      type: "guid",
      hidden: true,
      secret: true
    }
  }
};

// expose ajax functions
KingTable.Ajax = _ajax2.default;

// Pollute the window namespace with the KingTable object,
// this is intentional, so the users of the library that don't work with ES6, yet,
// can override its functions using: KingTable.prototype.propertyName = function something() {}
// Haters are gonna hate. But if you don't like, you can always create a custom build without following three lines! (MIT License)
if ((typeof window === "undefined" ? "undefined" : _typeof(window)) !== UNDEFINED) {
  window.KingTable = KingTable;
}

exports.default = KingTable;