"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _kingtable = require("../../scripts/tables/kingtable.builder");

var _kingtable2 = _interopRequireDefault(_kingtable);

var _textSlider = require("../../scripts/literature/text-slider");

var _exceptions = require("../../scripts/exceptions");

var _raise = require("../../scripts/raise");

var _raise2 = _interopRequireDefault(_raise);

var _utils = require("../../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _string = require("../../scripts/components/string");

var _string2 = _interopRequireDefault(_string);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * KingTable plain text builder.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Defines a table builder that renders tabular data in plain text,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * suitable for debug, unit tests, console output and plain text emails.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * https://github.com/RobertoPrevato/KingTable
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2018, Roberto Prevato
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * https://robertoprevato.github.io
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Licensed under the MIT license:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * http://www.opensource.org/licenses/MIT
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var SPACE = " ";
var RN = "\r\n";
var LINE_SEP = _string2.default.repeat("*", 65);

var KingTableTextBuilder = function (_KingTableBuilder) {
  _inherits(KingTableTextBuilder, _KingTableBuilder);

  /**
   * Creates a new instance of KingTableTextBuilder associated with the given table.
   */
  function KingTableTextBuilder(table) {
    _classCallCheck(this, KingTableTextBuilder);

    var _this = _possibleConstructorReturn(this, (KingTableTextBuilder.__proto__ || Object.getPrototypeOf(KingTableTextBuilder)).call(this, table));

    _this.slider = new _textSlider.TextSlider("....");
    _this.setListeners(table);
    return _this;
  }

  /**
   * Sets listeners for the given table.
   */


  _createClass(KingTableTextBuilder, [{
    key: "setListeners",
    value: function setListeners(table) {
      if (!table) return;
      var self = this;

      if (table.element && KingTableTextBuilder.options.handleLoadingInfo) {
        self.listenTo(table, "fetching:data", function () {
          self.loadingHandler(table);
        });
        self.listenTo(table, "fetched:data", function () {
          self.unsetLoadingHandler();
        });
        self.listenTo(table, "fetch:fail", function () {
          self.unsetLoadingHandler().display(table, self.errorView());
        });
        self.listenTo(table, "no-results", function () {
          self.unsetLoadingHandler().display(table, self.emptyView());
        });
      }
    }

    /**
     * Global options for every KingTableTextBuilder.
     */

  }, {
    key: "singleLine",


    /**
     * Returns an information for the table in a single line, including
     * table caption and pagination information, if available.
     */
    value: function singleLine(table, line) {
      return this.tabulate([[line]], [], _utils2.default.extend({
        caption: table.options.caption
      }, table.pagination.totalPageCount > 0 ? table.pagination.data() : null));
    }

    /**
     * Returns an error view.
     */

  }, {
    key: "errorView",
    value: function errorView() {
      var table = this.table;
      var reg = this.getReg();
      return this.singleLine(table, reg.errorFetchingData);
    }
  }, {
    key: "loadingHandler",
    value: function loadingHandler(table) {
      var self = this;
      self.unsetLoadingHandler();
      var slider = this.slider;
      var reg = this.getReg();
      var label = reg.loading + " ";

      var delayInfo = table.hasData() ? KingTableTextBuilder.options.loadInfoDelay : 0;
      // display a loading information, but only if waiting for more than n milliseconds
      var element = table.element;
      self.showLoadingTimeout = setTimeout(function () {
        if (!table.loading) {
          return self.unsetLoadingHandler();
        }
        var text = self.singleLine(table, label + slider.next());
        if (element) {
          element.innerHTML = text;
        }

        // set interval, to display a nice text animation while loading
        //
        self.loadingInterval = setInterval(function () {
          if (!table.loading) {
            return self.unsetLoadingHandler();
          }
          var text = self.singleLine(table, label + slider.next());
          element.innerHTML = text;
        }, 600);
      }, delayInfo);
    }
  }, {
    key: "unsetLoadingHandler",
    value: function unsetLoadingHandler() {
      clearInterval(this.loadingInterval);
      clearTimeout(this.showLoadingTimeout);
      this.loadingInterval = this.showLoadingTimeout = null;
      return this;
    }

    /**
     * Disposes of this KingTableTextBuilder.
     */

  }, {
    key: "dispose",
    value: function dispose() {
      var element = table.element;
      if (element) {
        element.innerHTML = "";
      }
      this.stopListening(this.table);
      this.table = null;
      this.slider = null;
    }

    /**
     * Displays a built table.
     */

  }, {
    key: "display",
    value: function display(table, built) {
      //
      // NB: aside from this piece of code, this class is abstracted
      // from DOM manipulation;
      // If a table has an element, assume that is a DOM element;
      //
      var element = table.element;
      if (element) {
        //
        // NB: this class does not set any event handler,
        // hence does not try to unset any event handler when removing an element.
        //
        // a custom event is fired, so the user of the library can unset any event added
        // by other means (e.g. vanilla JavaScript or jQuery)
        //
        table.emit("empty:element", element);
        while (element.hasChildNodes()) {
          element.removeChild(element.lastChild);
        }
        if (element.tagName != "PRE") {
          // append a PRE element
          var child = document.createElement("pre");
          element.appendChild(child);
          element = child;
        }
        element.innerHTML = built;
        //
        // TODO: add method to dispose the element for each view handler?
      }
    }

    /**
     * Builds the given instance of KingTable in plain text.
     */

  }, {
    key: "build",
    value: function build(table) {
      if (!table) table = this.table;
      var a = table.getData({
        optimize: true,
        format: true
      });
      if (!a || !a.length) {
        return this.display(table, this.emptyView());
      }
      var headers = a.shift();
      var built = this.tabulate(headers, a, _utils2.default.extend({
        caption: table.options.caption,
        dataAnchorTime: table.options.showAnchorTimestamp ? table.getFormattedAnchorTime() : null
      }, table.pagination.data()));
      //
      // display the table:
      //
      this.display(table, built);
    }
  }, {
    key: "emptyView",
    value: function emptyView() {
      var reg = this.getReg();
      return LINE_SEP + RN + reg.noData + RN + LINE_SEP;
    }
  }, {
    key: "paginationInfo",
    value: function paginationInfo(data) {
      var s = "",
          sep = " - ";
      var reg = this.getReg(),
          page = data.page,
          totalPageCount = data.totalPageCount,
          firstObjectNumber = data.firstObjectNumber,
          lastObjectNumber = data.lastObjectNumber,
          totalItemsCount = data.totalItemsCount,
          dataAnchorTime = data.dataAnchorTime,
          isNum = _utils2.default.isNumber;
      if (isNum(page)) {
        s += reg.page + SPACE + page;

        if (isNum(totalPageCount)) {
          s += SPACE + reg.of + SPACE + totalPageCount;
        }

        if (isNum(firstObjectNumber) && isNum(lastObjectNumber)) {
          s += sep + reg.results + _string2.default.format(" {0} - {1}", firstObjectNumber, lastObjectNumber);
          if (isNum(totalItemsCount)) {
            s += _string2.default.format(" " + reg.of + " {0}", totalItemsCount);
          }
        }
      }
      if (dataAnchorTime) {
        s += sep + (reg.anchorTime + " " + dataAnchorTime);
      }
      return s;
    }

    /**
     * Sanitizes a string for display in a plain text table.
     */

  }, {
    key: "checkValue",
    value: function checkValue(s) {
      if (!s) return "";
      if (typeof s != "string") s = s.toString();
      return s ? s.replace(/\r/g, "␍").replace(/\n/g, "␊") : "";
    }

    /**
     * Creates a plain text tabular representation of data, given a set of headers and rows of values.
     * NB: this function is not responsible of formatting values! e.g. create string representations of numbers or dates
     *
     * @example
     * // returns +-------+-------+-------+
                  |  id   | name  | value |
                  +-------+-------+-------+
                  | 1     | AAA   | A11   |
                  +-------+-------+-------+
                  | 2     | BBB   | B11   |
                  +-------+-------+-------+
                  | 3     | CCC   | C11   |
                  +-------+-------+-------+
                  | 4     | DDD   | D11   |
                  +-------+-------+-------+
        var a = new KingTableTextBuilder();
        a.tabulate(["id", "name", "value"],
        [
          [1, "AAA", "A11"],
          [2, "BBB", "B11"],
          [3, "CCC", "C11"],
          [4, "DDD", "D11"]
        ]);
     */

  }, {
    key: "tabulate",
    value: function tabulate(headers, rows, options) {
      var _this2 = this;

      if (!_utils2.default.isArray(headers)) (0, _exceptions.TypeException)("headers", "array");
      if (!rows) (0, _exceptions.TypeException)("rows", "array");
      if (_utils2.default.any(rows, function (x) {
        return !_utils2.default.isArray(x);
      })) (0, _exceptions.TypeException)("rows child", "array");
      var o = _utils2.default.extend({}, KingTableTextBuilder.options, options || {});
      var headersAlignment = o.headersAlignment,
          rowsAlignment = o.rowsAlignment,
          padding = o.padding,
          cornerChar = o.cornerChar,
          headerLineSeparator = o.headerLineSeparator,
          cellVerticalLine = o.cellVerticalLine,
          cellHorizontalLine = o.cellHorizontalLine,
          minCellWidth = o.minCellWidth,
          headerCornerChar = o.headerCornerChar;
      if (padding < 0) (0, _exceptions.OutOfRangeException)("padding", 0);
      var self = this;
      // validate the length of headers and of each row: it must be the same
      var headersLength = headers.length;
      if (!headersLength) (0, _exceptions.ArgumentException)("headers must contain at least one item");
      if (_utils2.default.any(rows, function (x) {
        x.length != headersLength;
      })) (0, _exceptions.ArgumentException)("each row must contain the same number of items");

      var s = "";

      // sanitize all values
      _utils2.default.reach(headers, function (x) {
        return _this2.checkValue(x);
      });
      _utils2.default.reach(rows, function (x) {
        return _this2.checkValue(x);
      });

      var valueLeftPadding = _string2.default.ofLength(SPACE, padding);
      padding = padding * 2;
      // for each column, get the cell width
      var cols = _utils2.default.cols([headers].concat(rows)),
          colsLength = _utils2.default.map(cols, function (x) {
        return Math.max(_utils2.default.max(x, function (y) {
          return y.length;
        }), minCellWidth) + padding;
      });
      // does the table contains a caption?
      var totalRowLength;
      var caption = o.caption,
          checkLength = 0;
      if (caption) {
        checkLength = padding + caption.length + 2;
      }

      // does option contains information about the pagination?
      var paginationInfo = self.paginationInfo(o);
      if (paginationInfo) {
        // is the pagination info bigger than whole row length?
        var pageInfoLength = padding + paginationInfo.length + 2;
        checkLength = Math.max(checkLength, pageInfoLength);
      }

      // check if the last column length should be adapted to the header length
      if (checkLength > 0) {
        totalRowLength = _utils2.default.sum(colsLength) + colsLength.length + 1;
        if (checkLength > totalRowLength) {
          var fix = checkLength - totalRowLength;
          colsLength[colsLength.length - 1] += fix;
          totalRowLength = checkLength;
        }
      }

      if (caption) {
        s += cellVerticalLine + valueLeftPadding + caption + _string2.default.ofLength(SPACE, totalRowLength - caption.length - 3) + cellVerticalLine + RN;
      }
      if (paginationInfo) {
        s += cellVerticalLine + valueLeftPadding + paginationInfo + _string2.default.ofLength(SPACE, totalRowLength - paginationInfo.length - 3) + cellVerticalLine + RN;
      }

      var headCellsSeps = _utils2.default.map(colsLength, function (l) {
        return _string2.default.ofLength(headerLineSeparator, l);
      }),
          cellsSeps = _utils2.default.map(colsLength, function (l) {
        return _string2.default.ofLength(cellHorizontalLine, l);
      });

      var headerLineSep = "";
      // add the first line
      _utils2.default.each(headers, function (x, i) {
        headerLineSep += headerCornerChar + headCellsSeps[i];
      });
      // add last vertical separator
      headerLineSep += headerCornerChar + RN;

      if (paginationInfo || caption) {
        s = headerLineSep + s;
      }
      s += headerLineSep;

      // add headers
      _utils2.default.each(headers, function (x, i) {
        s += cellVerticalLine + self.align(valueLeftPadding + x, colsLength[i], headersAlignment);
      });

      // add last vertical singleLineSeparator
      s += cellVerticalLine + RN;

      // add header separator
      s += headerLineSep;

      // build line separator
      var lineSep = "",
          i;
      for (i = 0; i < headersLength; i++) {
        lineSep += cornerChar + cellsSeps[i];
      }lineSep += cornerChar;

      // build rows
      var rowsLength = rows.length,
          j,
          row,
          value;
      for (i = 0; i < rowsLength; i++) {
        row = rows[i];
        for (j = 0; j < headersLength; j++) {
          value = row[j];
          s += cellVerticalLine + self.align(valueLeftPadding + value, colsLength[j], rowsAlignment);
        }
        s += cellVerticalLine + RN;
        s += lineSep + RN;
      }
      return s;
    }

    /**
     * Applies an alignment to the given text, using the given length and filler, by alignment code.
     */

  }, {
    key: "align",
    value: function align(text, length, alignment, filler) {
      if (!filler) filler = SPACE;
      if (!alignment) (0, _exceptions.ArgumentNullException)("alignment");
      switch (alignment) {
        case "c":
        case "center":
          return _string2.default.center(text, length, filler);
        case "l":
        case "left":
          return _string2.default.ljust(text, length, filler);
          break;
        case "r":
        case "right":
          return _string2.default.rjust(text, length, filler);
        default:
          (0, _exceptions.ArgumentException)("alignment: " + alignment);
      }
    }
  }], [{
    key: "options",
    get: function get() {
      return {
        headerLineSeparator: "=",
        headerCornerChar: "=",
        cornerChar: "+",
        headersAlignment: "l",
        rowsAlignment: "l",
        padding: 1,
        cellVerticalLine: "|",
        cellHorizontalLine: "-",
        minCellWidth: 0,
        handleLoadingInfo: true, // whether to display loading information (suitable for console applications)
        loadInfoDelay: 500 // how many milliseconds should wait, before displaying the "Loading..." information
      };
    }
  }]);

  return KingTableTextBuilder;
}(_kingtable2.default);

exports.default = KingTableTextBuilder;