"use strict";

var _utils = require("../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _ajax = require("../scripts/data/ajax");

var _ajax2 = _interopRequireDefault(_ajax);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Tests for AJAX utilities.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
describe("AJAX utilities", function () {

  it("must allow to produce query strings (single parameter)", function () {
    var a = {
      "search": "Hello"
    };

    var qs = _ajax2.default.createQs(a);
    expect(qs).toEqual("search=Hello");
  });

  it("must allow to produce query strings (multiple parameters)", function () {
    var a = {
      "search": "Hello",
      "page": 1,
      "size": 30
    };

    var qs = _ajax2.default.createQs(a);
    expect(qs).toEqual("page=1&search=Hello&size=30");
  });

  it("must allow to produce query strings, ignoring null values (multiple parameters)", function () {
    var a = {
      "search": null,
      "page": 1,
      "size": 30,
      "moo": undefined,
      "foo": ""
    };

    var qs = _ajax2.default.createQs(a);
    expect(qs).toEqual("page=1&size=30");
  });

  it("must allow to produce query strings, handling null and undefined values", function () {
    expect(_ajax2.default.createQs(null)).toEqual("", "null must produce an empty query string");

    expect(_ajax2.default.createQs(undefined)).toEqual("", "undefined must produce an empty query string");
  });
});