"use strict";

var _csv = require("../scripts/data/csv");

var _csv2 = _interopRequireDefault(_csv);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BOM = "\uFEFF"; /**
                     * Tests for csv utility functions.
                     * https://github.com/RobertoPrevato/KingTable
                     *
                     * Copyright 2018, Roberto Prevato
                     * https://robertoprevato.github.io
                     *
                     * Licensed under the MIT license:
                     * http://www.opensource.org/licenses/MIT
                     */


describe("CSV helper functions", function () {
  it("must allow to serialize collections in CSV", function () {
    var a = [["id", "firstname", "lastname"], [1, "Giulio", "Cesare"]];
    var b = _csv2.default.serialize(a);

    var expected = BOM + "id,firstname,lastname\n1,Giulio,Cesare";
    expect(b).toEqual(expected);
  });

  it("must handle values containing commas", function () {
    var a = [["id", "model", "something"], [1, "A31, KOP", "Hello World"]];
    var b = _csv2.default.serialize(a);

    var expected = BOM + "id,model,something\n1,\"A31, KOP\",Hello World";
    expect(b).toEqual(expected);
  });

  it("must handle values containing commas and double quotes", function () {
    var a = [["id", "model", "something"], [1, "A31, \"KOP\"", "Hello World"]];
    var b = _csv2.default.serialize(a);
    var expected = BOM + "id,model,something\n1,\"A31, \"\"KOP\"\"\",Hello World";
    expect(b).toEqual(expected);
  });

  it("must allow to use custom separators", function () {
    var a = [["id", "model", "something"], [1, "A31, KOP", "Hello World"]];
    var b = _csv2.default.serialize(a, { separator: "|" });

    var expected = BOM + "id|model|something\n1|A31, KOP|Hello World";
    expect(b).toEqual(expected);
  });

  it("must allow to remove BOM", function () {
    var a = [["id", "model", "something"], [1, "A31, KOP", "Hello World"]];
    var b = _csv2.default.serialize(a, { addBom: false, separator: "|" });

    var expected = "id|model|something\n1|A31, KOP|Hello World";
    expect(b).toEqual(expected);
  });

  it("must allow to include separator, for MS Excel", function () {
    var a = [["id", "firstname", "lastname"], [1, "Giulio", "Cesare"]];
    var b = _csv2.default.serialize(a, { addSeparatorLine: true });

    var expected = BOM + "id,firstname,lastname\n1,Giulio,Cesare\n\t,";
    expect(b).toEqual(expected);
  });
});