"use strict";

var _json = require("../scripts/data/json");

var _json2 = _interopRequireDefault(_json);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("JSON proxy functions", function () {
  it("must parse dates in ISO format automatically", function () {
    var a = '{"a":"2017-02-08T15:32:27.118Z"}';
    var b = _json2.default.parse(a);

    expect(b.a instanceof Date).toEqual(true, "a string in ISO format should be parsed automatically as date");
    expect(b.a.getFullYear()).toEqual(2017);
    expect(b.a.getMonth()).toEqual(1);
    expect(b.a.getDate()).toEqual(8);
    expect(b.a.getUTCHours()).toEqual(15);
    expect(b.a.getUTCMinutes()).toEqual(32);
  });

  it("must parse dates not in ISO format automatically (HH:mm:ss)", function () {
    var a = '{"a":"2017-02-08 15:32:15"}';
    var b = _json2.default.parse(a);

    expect(b.a instanceof Date).toEqual(true, "a string in ISO format should be parsed automatically as date");
    expect(b.a.getFullYear()).toEqual(2017);
    expect(b.a.getMonth()).toEqual(1);
    expect(b.a.getDate()).toEqual(8);
    expect(b.a.getHours()).toEqual(15);
    expect(b.a.getMinutes()).toEqual(32);
    expect(b.a.getSeconds()).toEqual(15);
  });

  it("must parse dates not in ISO format automatically (HH:mm)", function () {
    var a = '{"a":"2017-02-08 15:32"}';
    var b = _json2.default.parse(a);

    expect(b.a instanceof Date).toEqual(true, "a string in ISO format should be parsed automatically as date");
    expect(b.a.getFullYear()).toEqual(2017);
    expect(b.a.getMonth()).toEqual(1);
    expect(b.a.getDate()).toEqual(8);
    expect(b.a.getHours()).toEqual(15);
    expect(b.a.getMinutes()).toEqual(32);
    expect(b.a.getSeconds()).toEqual(0);
  });

  it("must parse dates not in ISO format automatically (HH)", function () {
    var a = '{"a":"2017-02-08 15"}';
    var b = _json2.default.parse(a);

    expect(b.a instanceof Date).toEqual(true, "a string in ISO format should be parsed automatically as date");
    expect(b.a.getFullYear()).toEqual(2017);
    expect(b.a.getMonth()).toEqual(1);
    expect(b.a.getDate()).toEqual(8);
    expect(b.a.getHours()).toEqual(15);
    expect(b.a.getMinutes()).toEqual(0);
    expect(b.a.getSeconds()).toEqual(0);
  });

  it("must include properties with undefined values in serialized strings", function () {
    var a = { "someProperty": undefined };
    var b = _json2.default.compose(a);

    expect(b.indexOf("someProperty") != -1).toEqual(true, "a serialized JSON object must include undefined properties");
    expect(b.indexOf("someProperty\":null") != -1).toEqual(true, "undefined values are serialized as null");
  });
}); /**
     * Tests for json proxy functions.
     * https://github.com/RobertoPrevato/KingTable
     *
     * Copyright 2018, Roberto Prevato
     * https://robertoprevato.github.io
     *
     * Licensed under the MIT license:
     * http://www.opensource.org/licenses/MIT
     */