"use strict";

var _kingtableHtml = require("../scripts/tables/kingtable.html.builder");

var _kingtableHtml2 = _interopRequireDefault(_kingtableHtml);

var _colors = require("../tests/data/colors");

var _colors2 = _interopRequireDefault(_colors);

var _latestScores = require("../tests/data/latest-scores");

var _latestScores2 = _interopRequireDefault(_latestScores);

var _utils = require("../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Tests for KingTableHtmlBuilder class.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
describe("KingTableHtmlBuilder", function () {

  it("must produce html fragments for highlighted texts", function () {
    var a = new _kingtableHtml2.default();
    var text = "Hello World";
    var pattern = /(world)/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("Hello <span class=\"kt-search-highlight\">World</span>");
  });

  it("must produce html fragments for highlighted texts (beginning of string)", function () {
    var a = new _kingtableHtml2.default();
    var text = "Hello World";
    var pattern = /(hello)/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("<span class=\"kt-search-highlight\">Hello</span> World");
  });

  it("must produce html fragments for highlighted texts, with multiple matches (beginning of string)", function () {
    var a = new _kingtableHtml2.default();
    var text = "Hello World hell-o)))-ween";
    var pattern = /hell/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("<span class=\"kt-search-highlight\">Hell</span>o World <span class=\"kt-search-highlight\">hell</span>-o)))-ween");
  });

  it("must produce html fragments for highlighted texts, with multiple matches", function () {
    var a = new _kingtableHtml2.default();
    var text = "Hello World Kitty world";
    var pattern = /(world)/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("Hello <span class=\"kt-search-highlight\">World</span> Kitty <span class=\"kt-search-highlight\">world</span>");
  });

  it("must produce html fragments for highlighted texts, with multiple matches (no group)", function () {
    var a = new _kingtableHtml2.default();
    var text = "Hello World Kitty world";
    var pattern = /world/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("Hello <span class=\"kt-search-highlight\">World</span> Kitty <span class=\"kt-search-highlight\">world</span>");
  });

  it("must escape html characters when producing html fragments for highlighted texts, of highlighted string", function () {
    var a = new _kingtableHtml2.default();
    var text = "Hello <script>console.error('This should not happen')</script>World";
    var pattern = /(<script>console.error\('This should not happen'\)<\/script>)/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("Hello <span class=\"kt-search-highlight\">&lt;script&gt;console.error(&#039;This should not happen&#039;)&lt;/script&gt;</span>World");
  });

  it("must escape html characters when producing html fragments for highlighted texts, multiple matches", function () {
    var a = new _kingtableHtml2.default();
    var text = "Hello <script>alert('foo!')</script> <script>console.error('This should not happen')</script>World";
    var pattern = /(<script>console.error\('This should not happen'\)<\/script>)/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("Hello &lt;script&gt;alert(&#039;foo!&#039;)&lt;/script&gt; <span class=\"kt-search-highlight\">&lt;script&gt;console.error(&#039;This should not happen&#039;)&lt;/script&gt;</span>World");
  });

  it("must produce html fragments for highlighted texts (without matches)", function () {
    var a = new _kingtableHtml2.default();
    var text = "Hello World";
    var pattern = /(without)/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("Hello World");
  });

  it("must escape html fragments", function () {
    var a = new _kingtableHtml2.default();
    var text = "<script>alert('Foo')</script>";
    var pattern = /scri/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("&lt;<span class=\"kt-search-highlight\">scri</span>pt&gt;alert(&#039;Foo&#039;)&lt;/<span class=\"kt-search-highlight\">scri</span>pt&gt;");
  });

  it("must support diacritics in highlights", function () {
    var a = new _kingtableHtml2.default();
    var text = "Błażkiewicz";
    var pattern = /laz/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("B<span class=\"kt-search-highlight\">\u0142a\u017C</span>kiewicz");
  });

  it("must support diacritics, two outside highlight", function () {
    var a = new _kingtableHtml2.default();
    var text = "Barłysław";
    var pattern = /bar/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("<span class=\"kt-search-highlight\">Bar</span>\u0142ys\u0142aw");
  });

  it("must support diacritics, many outside highlight", function () {
    var a = new _kingtableHtml2.default();
    var text = "Barłysławłłł";
    var pattern = /bar/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("<span class=\"kt-search-highlight\">Bar</span>\u0142ys\u0142aw\u0142\u0142\u0142");
  });

  it("must support diacritics, properly highlight Chinese characters", function () {
    var a = new _kingtableHtml2.default();
    var text = "2017年12月31日";
    var pattern = /日/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("2017\u5E7412\u670831<span class=\"kt-search-highlight\">\u65E5</span>");
  });

  it("must support diacritics, properly highlight non-Chinese characters in Chinese characters string", function () {
    var a = new _kingtableHtml2.default();
    var text = "2017年12月31日";
    var pattern = /12/gim;

    var html = a.highlight(text, pattern);
    expect(html).toEqual("2017\u5E74<span class=\"kt-search-highlight\">12</span>\u670831\u65E5");
  });
});