"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /**
                                                                                                                                                                                                                                                                               * Tests for LRU caching mechanism.
                                                                                                                                                                                                                                                                               * https://github.com/RobertoPrevato/KingTable
                                                                                                                                                                                                                                                                               *
                                                                                                                                                                                                                                                                               * Copyright 2018, Roberto Prevato
                                                                                                                                                                                                                                                                               * https://robertoprevato.github.io
                                                                                                                                                                                                                                                                               *
                                                                                                                                                                                                                                                                               * Licensed under the MIT license:
                                                                                                                                                                                                                                                                               * http://www.opensource.org/licenses/MIT
                                                                                                                                                                                                                                                                               */


var _utils = require("../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _lru = require("../scripts/data/lru");

var _lru2 = _interopRequireDefault(_lru);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("LRU cache", function () {

  it("must store least recently used items by key", function () {
    var key = "a";
    sessionStorage.removeItem(key);

    for (var i = 0; i < 10; i++) {
      _lru2.default.set(key, i);
    }

    var val = _lru2.default.get(key);
    expect(val instanceof Array).toEqual(true, "getter with only key must return an array of cached items");
    _utils2.default.sortNums(val);
    expect(val[0]).toEqual(0);
    expect(val[1]).toEqual(1);
    expect(val[2]).toEqual(2);
    expect(val[3]).toEqual(3);
    expect(val[4]).toEqual(4);
    expect(val[5]).toEqual(5);
    expect(val[6]).toEqual(6);
    expect(val[7]).toEqual(7);
    expect(val[8]).toEqual(8);
    expect(val[9]).toEqual(9);
  });

  it("must allow to discard older items", function () {
    var key = "b";
    sessionStorage.removeItem(key);
    var maxSize = 5;

    for (var i = 0; i < 10; i++) {
      _lru2.default.set(key, i, maxSize);
    }

    var val = _lru2.default.get(key);
    expect(val instanceof Array).toEqual(true, "getter with only key must return an array of cached items");
    _utils2.default.sortNums(val);
    expect(val.length).toEqual(maxSize);
    expect(val[0]).toEqual(5);
    expect(val[1]).toEqual(6);
    expect(val[2]).toEqual(7);
    expect(val[3]).toEqual(8);
    expect(val[4]).toEqual(9);

    _lru2.default.set(key, 20, maxSize);
    var val = _lru2.default.get(key);
    expect(val instanceof Array).toEqual(true, "getter with only key must return an array of cached items");
    _utils2.default.sortNums(val);
    expect(val.length).toEqual(maxSize);
    expect(val[0]).toEqual(6);
    expect(val[1]).toEqual(7);
    expect(val[2]).toEqual(8);
    expect(val[3]).toEqual(9);
    expect(val[4]).toEqual(20);
  });

  it("must retrieve single items by key and condition", function () {
    var key = "c";
    sessionStorage.removeItem(key);

    for (var i = 0; i < 10; i++) {
      _lru2.default.set(key, i);
    }

    var val = _lru2.default.get(key, function (x) {
      return x === 6;
    });
    expect(val instanceof Array).toEqual(false, "getter with key and condition must return an exact item");
    expect(val).toEqual(6);
  });

  it("must remove items by key and condition", function () {
    var key = "d";
    sessionStorage.removeItem(key);

    for (var i = 0; i < 5; i++) {
      _lru2.default.set(key, i);
    }

    var val = _lru2.default.get(key);
    expect(val instanceof Array).toEqual(true, "getter with only key must return an array of cached items");
    expect(val.length).toEqual(5);
    _utils2.default.sortNums(val);
    expect(val[0]).toEqual(0);
    expect(val[1]).toEqual(1);
    expect(val[2]).toEqual(2);
    expect(val[3]).toEqual(3);
    expect(val[4]).toEqual(4);

    _lru2.default.remove(key, function (x) {
      return x >= 3;
    });
    val = _lru2.default.get(key);
    expect(val instanceof Array).toEqual(true, "getter with only key must return an array of cached items");
    expect(val.length).toEqual(3);
    _utils2.default.sortNums(val);
    expect(val[0]).toEqual(0);
    expect(val[1]).toEqual(1);
    expect(val[2]).toEqual(2);
  });

  it("must allow to retrieve single objects by key and condition", function () {
    var key = "e";
    sessionStorage.removeItem(key);

    for (var i = 0; i < 10; i++) {
      _lru2.default.set(key, { val: i });
    }

    var o = _lru2.default.get(key, function (x) {
      return x.val === 6;
    });
    expect(o instanceof Array).toEqual(false, "getter with key and condition must return an exact item");
    expect(o.val).toEqual(6);
  });

  it("must allow to return items with their caching details", function () {
    var key = "f";
    sessionStorage.removeItem(key);

    for (var i = 0; i < 10; i++) {
      _lru2.default.set(key, { val: i });
    }

    var o = _lru2.default.get(key, function (x) {
      return x.val === 6;
    }, true);
    expect(o instanceof Array).toEqual(false, "getter with key and condition must return an exact item");
    expect(_typeof(o.ts)).toEqual("number", "lru getter must return details with caching ts");
    expect(o.data.val).toEqual(6, "lru getter must return details with data in 'data' property");
  });
});