"use strict";

var _utils = require("../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _html = require("../scripts/data/html");

var _kingtableMenu = require("../scripts/menus/kingtable.menu.html");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("Function menuItemBuilder", function () {

  it("must allow to generate HTML for a menu item with span", function () {
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Lorem ipsum"
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<li><span tabindex=\"0\">Lorem ipsum</span></li>");
  });

  it("must allow to generate HTML for a menu item with span and id for wrapper", function () {
    var el = (0, _kingtableMenu.menuItemBuilder)({
      id: "lorem",
      name: "Lorem ipsum"
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<li id=\"lorem\"><span tabindex=\"0\">Lorem ipsum</span></li>");
  });

  it("must allow to generate HTML for a menu item with span and id for wrapper and child element", function () {
    var el = (0, _kingtableMenu.menuItemBuilder)({
      id: "lorem",
      name: "Lorem ipsum",
      attr: {
        id: "ipsum"
      }
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<li id=\"lorem\"><span tabindex=\"0\" id=\"ipsum\">Lorem ipsum</span></li>");
  });

  it("must allow to generate HTML for a menu item with span and class", function () {
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Lorem ipsum",
      attr: {
        "class": "lorem-ipsum"
      }
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<li><span tabindex=\"0\" class=\"lorem-ipsum\">Lorem ipsum</span></li>");
  });

  it("must allow to generate HTML for a menu item with span and class using `css` property", function () {
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Lorem ipsum",
      attr: {
        css: "lorem-ipsum"
      }
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<li><span tabindex=\"0\" class=\"lorem-ipsum\">Lorem ipsum</span></li>");
  });

  it("must allow to generate HTML for a menu item with anchor", function () {
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Lorem ipsum",
      href: "#/lorem-ipsum"
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<li><a href=\"#/lorem-ipsum\">Lorem ipsum</a></li>");
  });

  it("must allow to generate HTML for a menu item with anchor and class", function () {
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Lorem ipsum",
      href: "#/lorem-ipsum",
      attr: {
        "class": "lorem-ipsum"
      }
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<li><a href=\"#/lorem-ipsum\" class=\"lorem-ipsum\">Lorem ipsum</a></li>");
  });

  it("must allow to generate HTML for a menu item with anchor and class using `css` property", function () {
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Lorem ipsum",
      href: "#/lorem-ipsum",
      attr: {
        css: "lorem-ipsum"
      }
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<li><a href=\"#/lorem-ipsum\" class=\"lorem-ipsum\">Lorem ipsum</a></li>");
  });

  it("must allow to generate HTML for a menu item with label and checkbox", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Display name",
      type: "checkbox",
      attr: { // any attribute to add to the generated html
        "name": "input-attribute", // input attribute
        "class": "some-item"
      }
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    // indentation is used to improve readability of text;
    var html = el.toString(2, "_");
    expect(html).toEqual("<li>\n__<input name=\"input-attribute\" class=\"some-item\" id=\"mnck-0\" type=\"checkbox\" />\n__<label for=\"mnck-0\">\n____Display name\n__</label>\n</li>\n");
  });

  it("must allow to generate HTML for a menu item with label and checked checkbox", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Display name",
      type: "checkbox",
      checked: true,
      attr: { // any attribute to add to the generated html
        "name": "input-attribute", // input attribute
        "class": "some-item"
      }
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    // indentation is used to improve readability of text;
    var html = el.toString(2, "_");
    expect(html).toEqual("<li>\n__<input name=\"input-attribute\" class=\"some-item\" id=\"mnck-0\" type=\"checkbox\" checked />\n__<label for=\"mnck-0\">\n____Display name\n__</label>\n</li>\n");
  });

  it("must allow to generate HTML for a menu item with label and radio", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuItemBuilder)({
      name: "Display name",
      checked: true,
      type: "radio",
      value: "some-value", // this is the radio value; and is required
      attr: { // any attribute to add to the generated html
        "name": "input-attribute", // input attribute
        "class": "some-item"
      }
    });
    expect(el instanceof _html.VHtmlElement).toEqual(true);
    // indentation is used to improve readability of text;
    var html = el.toString(2, "_");
    expect(html).toEqual("<li>\n__<input name=\"input-attribute\" class=\"some-item\" id=\"mnrd-0\" type=\"radio\" checked value=\"some-value\" />\n__<label for=\"mnrd-0\">\n____Display name\n__</label>\n</li>\n");
  });
}); /**
     * Tests for menus HTML functions.
     * https://github.com/RobertoPrevato/KingTable
     *
     * Copyright 2018, Roberto Prevato
     * https://robertoprevato.github.io
     *
     * Licensed under the MIT license:
     * http://www.opensource.org/licenses/MIT
     */


describe("Function menuBuilder", function () {

  it("must allow to generate HTML for a menu without items", function () {
    var el = (0, _kingtableMenu.menuBuilder)({
      name: "Lorem ipsum"
    });
    expect(el instanceof _html.VWrapperElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<ul class=\"ug-menu\"></ul>");
  });

  it("must allow to generate HTML for a menu with one span item", function () {
    var el = (0, _kingtableMenu.menuBuilder)({
      name: "Lorem ipsum",
      items: [{
        name: "Lorem ipsum"
      }]
    });
    expect(el instanceof _html.VWrapperElement).toEqual(true);
    var html = el.toString();
    expect(html).toEqual("<ul class=\"ug-menu\"><li><span tabindex=\"0\">Lorem ipsum</span></li></ul>");
  });

  it("must allow to generate HTML for a menu with one span item and one checkbox item", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuBuilder)({
      name: "Lorem ipsum",
      items: [{
        name: "Lorem ipsum"
      }, {
        name: "Display name",
        checked: true,
        type: "checkbox",
        attr: { // any attribute to add to the generated html
          "name": "input-attribute", // input attribute
          "class": "some-item"
        }
      }]
    });
    expect(el instanceof _html.VWrapperElement).toEqual(true);
    var html = el.toString(2, "_");
    expect(html).toEqual("<ul class=\"ug-menu\">\n__<li>\n____<span tabindex=\"0\">\n______Lorem ipsum\n____</span>\n__</li>\n__<li>\n____<input name=\"input-attribute\" class=\"some-item\" id=\"mnck-0\" type=\"checkbox\" checked />\n____<label for=\"mnck-0\">\n______Display name\n____</label>\n__</li>\n</ul>\n");
  });

  it("must allow to generate HTML for a menu with one span item and one radio item", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuBuilder)({
      name: "Lorem ipsum",
      items: [{
        name: "Lorem ipsum"
      }, {
        name: "Display name",
        checked: true,
        type: "radio",
        value: "some-value", // this is the radio value; and is required
        attr: { // any attribute to add to the generated html
          "name": "input-attribute", // input attribute
          "class": "some-item"
        }
      }]
    });
    expect(el instanceof _html.VWrapperElement).toEqual(true);
    var html = el.toString(2, "_");
    expect(html).toEqual("<ul class=\"ug-menu\">\n__<li>\n____<span tabindex=\"0\">\n______Lorem ipsum\n____</span>\n__</li>\n__<li>\n____<input name=\"input-attribute\" class=\"some-item\" id=\"mnrd-0\" type=\"radio\" checked value=\"some-value\" />\n____<label for=\"mnrd-0\">\n______Display name\n____</label>\n__</li>\n</ul>\n");
  });

  it("must allow to generate HTML for a menu with many radio items, with unique id", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuBuilder)({
      name: "Lorem ipsum",
      items: [{
        name: "Cats",
        checked: true,
        type: "radio",
        value: "cats", // this is the radio value; and is required
        attr: { // any attribute to add to the generated html
          "name": "favorite-animals", // input attribute
          "class": "some-item"
        }
      }, {
        name: "Dogs",
        checked: true,
        type: "radio",
        value: "dogs", // this is the radio value; and is required
        attr: { // any attribute to add to the generated html
          "name": "favorite-animals", // input attribute
          "class": "some-item"
        }
      }, {
        name: "Snakes",
        checked: true,
        type: "radio",
        value: "snakes", // this is the radio value; and is required
        attr: { // any attribute to add to the generated html
          "name": "favorite-animals", // input attribute
          "class": "some-item"
        }
      }]
    });
    expect(el instanceof _html.VWrapperElement).toEqual(true);
    var html = el.toString(2, "_");
    expect(html).toEqual("<ul class=\"ug-menu\">\n__<li>\n____<input name=\"favorite-animals\" class=\"some-item\" id=\"mnrd-0\" type=\"radio\" checked value=\"cats\" />\n____<label for=\"mnrd-0\">\n______Cats\n____</label>\n__</li>\n__<li>\n____<input name=\"favorite-animals\" class=\"some-item\" id=\"mnrd-1\" type=\"radio\" checked value=\"dogs\" />\n____<label for=\"mnrd-1\">\n______Dogs\n____</label>\n__</li>\n__<li>\n____<input name=\"favorite-animals\" class=\"some-item\" id=\"mnrd-2\" type=\"radio\" checked value=\"snakes\" />\n____<label for=\"mnrd-2\">\n______Snakes\n____</label>\n__</li>\n</ul>\n");
  });

  it("must allow to generate HTML for a menu with many checkbox items, with unique id", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuBuilder)({
      items: [{
        name: "Cats",
        checked: true,
        type: "checkbox",
        attr: { // any attribute to add to the generated html
          "name": "cats", // input attribute
          "class": "some-item"
        }
      }, {
        name: "Dogs",
        checked: true,
        type: "checkbox",
        attr: { // any attribute to add to the generated html
          "name": "dogs", // input attribute
          "class": "some-item"
        }
      }, {
        name: "Snakes",
        checked: true,
        type: "checkbox",
        attr: { // any attribute to add to the generated html
          "name": "snakes", // input attribute
          "class": "some-item"
        }
      }, {
        name: "Tigers",
        checked: true,
        type: "checkbox",
        attr: { // any attribute to add to the generated html
          "name": "tigers", // input attribute
          "class": "some-item"
        }
      }]
    });
    expect(el instanceof _html.VWrapperElement).toEqual(true);
    var html = el.toString(2, "_");
    expect(html).toEqual("<ul class=\"ug-menu\">\n__<li>\n____<input name=\"cats\" class=\"some-item\" id=\"mnck-0\" type=\"checkbox\" checked />\n____<label for=\"mnck-0\">\n______Cats\n____</label>\n__</li>\n__<li>\n____<input name=\"dogs\" class=\"some-item\" id=\"mnck-1\" type=\"checkbox\" checked />\n____<label for=\"mnck-1\">\n______Dogs\n____</label>\n__</li>\n__<li>\n____<input name=\"snakes\" class=\"some-item\" id=\"mnck-2\" type=\"checkbox\" checked />\n____<label for=\"mnck-2\">\n______Snakes\n____</label>\n__</li>\n__<li>\n____<input name=\"tigers\" class=\"some-item\" id=\"mnck-3\" type=\"checkbox\" checked />\n____<label for=\"mnck-3\">\n______Tigers\n____</label>\n__</li>\n</ul>\n");
  });

  it("must allow to generate HTML for a menu with submenus", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuBuilder)({
      items: [{
        name: "Something",
        menu: {
          id: "some-menu",
          items: [{
            name: "Lorem"
          }, {
            name: "Ipsum"
          }, {
            name: "Dolor",
            menu: {
              items: [{
                name: "Sit"
              }, {
                name: "Amet"
              }, {
                name: "Consectetur"
              }]
            }
          }]
        }
      }]
    });
    expect(el instanceof _html.VWrapperElement).toEqual(true);
    var html = el.toString(2, "_");
    expect(html).toEqual("<ul class=\"ug-menu\">\n__<li class=\"ug-submenu\">\n____<span tabindex=\"0\">\n______Something\n______<span class=\"oi\" data-glyph=\"caret-right\"></span>\n____</span>\n____<ul id=\"some-menu\" class=\"ug-menu\">\n______<li>\n________<span tabindex=\"0\">\n__________Lorem\n________</span>\n______</li>\n______<li>\n________<span tabindex=\"0\">\n__________Ipsum\n________</span>\n______</li>\n______<li class=\"ug-submenu\">\n________<span tabindex=\"0\">\n__________Dolor\n__________<span class=\"oi\" data-glyph=\"caret-right\"></span>\n________</span>\n________<ul class=\"ug-menu\">\n__________<li>\n____________<span tabindex=\"0\">\n______________Sit\n____________</span>\n__________</li>\n__________<li>\n____________<span tabindex=\"0\">\n______________Amet\n____________</span>\n__________</li>\n__________<li>\n____________<span tabindex=\"0\">\n______________Consectetur\n____________</span>\n__________</li>\n________</ul>\n______</li>\n____</ul>\n__</li>\n</ul>\n");
  });

  it("must allow to generate HTML for a menu with submenus and checkboxes", function () {
    _utils2.default.resetSeed();
    var el = (0, _kingtableMenu.menuBuilder)({
      items: [{
        name: "Something",
        menu: {
          id: "some-menu",
          items: [{
            name: "Lorem"
          }, {
            name: "Ipsum"
          }, {
            name: "Animals invited in the temple",
            menu: {
              items: [{
                name: "Cats",
                checked: false,
                type: "checkbox",
                attr: { // any attribute to add to the generated html
                  "name": "cats", // input attribute
                  "class": "some-item"
                }
              }, {
                name: "Dogs",
                checked: false,
                type: "checkbox",
                attr: { // any attribute to add to the generated html
                  "name": "dogs", // input attribute
                  "class": "some-item"
                }
              }, {
                name: "Snakes",
                checked: false,
                type: "checkbox",
                attr: { // any attribute to add to the generated html
                  "name": "snakes", // input attribute
                  "class": "some-item"
                }
              }, {
                name: "Tigers",
                checked: true,
                type: "checkbox",
                attr: { // any attribute to add to the generated html
                  "name": "tigers", // input attribute
                  "class": "some-item"
                }
              }]
            }
          }]
        }
      }]
    });
    expect(el instanceof _html.VWrapperElement).toEqual(true);
    var html = el.toString(2, "_");
    expect(html).toEqual("<ul class=\"ug-menu\">\n__<li class=\"ug-submenu\">\n____<span tabindex=\"0\">\n______Something\n______<span class=\"oi\" data-glyph=\"caret-right\"></span>\n____</span>\n____<ul id=\"some-menu\" class=\"ug-menu\">\n______<li>\n________<span tabindex=\"0\">\n__________Lorem\n________</span>\n______</li>\n______<li>\n________<span tabindex=\"0\">\n__________Ipsum\n________</span>\n______</li>\n______<li class=\"ug-submenu\">\n________<span tabindex=\"0\">\n__________Animals invited in the temple\n__________<span class=\"oi\" data-glyph=\"caret-right\"></span>\n________</span>\n________<ul class=\"ug-menu\">\n__________<li>\n____________<input name=\"cats\" class=\"some-item\" id=\"mnck-0\" type=\"checkbox\" />\n____________<label for=\"mnck-0\">\n______________Cats\n____________</label>\n__________</li>\n__________<li>\n____________<input name=\"dogs\" class=\"some-item\" id=\"mnck-1\" type=\"checkbox\" />\n____________<label for=\"mnck-1\">\n______________Dogs\n____________</label>\n__________</li>\n__________<li>\n____________<input name=\"snakes\" class=\"some-item\" id=\"mnck-2\" type=\"checkbox\" />\n____________<label for=\"mnck-2\">\n______________Snakes\n____________</label>\n__________</li>\n__________<li>\n____________<input name=\"tigers\" class=\"some-item\" id=\"mnck-3\" type=\"checkbox\" checked />\n____________<label for=\"mnck-3\">\n______________Tigers\n____________</label>\n__________</li>\n________</ul>\n______</li>\n____</ul>\n__</li>\n</ul>\n");
  });
});