"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Mocks, by monkey patching, the console object for unit tests.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */

/**
 * @example
   var mock = new MockConsole()

   mock.fire(() => {
     console.error("foo")
   })

   expect(mock.log[0]).toEqual("ERROR: foo")
   // dispose the mock console
   mock.dispose()
 */

var MockConsole = function () {
  function MockConsole() {
    var verbose = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

    _classCallCheck(this, MockConsole);

    this.log = [];
    this.setup();
    this.verbose = verbose;
  }

  _createClass(MockConsole, [{
    key: "setup",
    value: function setup() {
      var self = this;
      var names = "log info error".split(" ");
      var original = {};
      for (var i = 0, l = names.length; i < l; i++) {
        var name = names[i];
        original[name] = console[name];

        console[name] = function (a) {
          if (self.verbose) {
            original[name](a);
          }
          self.log.push(name.toUpperCase() + ": " + a);
        };
      }
      self.original = original;
    }
  }, {
    key: "dispose",
    value: function dispose() {
      var original = this.original;
      for (var x in original) {
        console[x] = original[x];
      }
    }
  }, {
    key: "fire",
    value: function fire(fn) {
      var eatExceptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

      // mock global console object
      var self = this;
      try {
        fn();
      } catch (ex) {
        if (!eatExceptions) {
          throw ex;
        }
      }
    }
  }]);

  return MockConsole;
}();

exports.MockConsole = MockConsole;