"use strict";

var _utils = require("../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _objectAnalyzer = require("../scripts/data/object-analyzer");

var _objectAnalyzer2 = _interopRequireDefault(_objectAnalyzer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Tests for Analyzer class.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
var a = new _objectAnalyzer2.default();

describe("Objects Analyzer class", function () {
  it("must describe an object structure", function () {
    var o = {
      a: "Hello",
      b: true,
      c: /aaa/,
      d: 300,
      e: 45.56
    };

    var schema = a.describe(o);
    expect(_utils2.default.equal(schema, {
      "a": "string",
      "b": "boolean",
      "c": "regex",
      "d": "number",
      "e": "number"
    })).toEqual(true, "the properties contain values of asserted types");
  });

  it("must describe the structure of list items", function () {
    var data = [{
      age: 30,
      name: "Roberto",
      status: "M",
      gender: "M"
    }];

    var schema = a.describeList(data, { limit: 1 });
    expect(_utils2.default.equal(schema, {
      "age": "number",
      "name": "string",
      "gender": "string",
      "status": "string"
    })).toEqual(true);
  });

  it("must describe the structure of list items with nullable properties", function () {
    var data = [{
      age: undefined,
      name: "AA",
      status: "M",
      gender: null
    }, {
      age: undefined,
      name: "BB",
      status: "M",
      gender: null
    }, {
      age: 30,
      name: "CC",
      status: "M",
      gender: "M"
    }];

    var schema = a.describeList(data);
    expect(_utils2.default.equal(schema, {
      "age": "number",
      "name": "string",
      "gender": "string",
      "status": "string"
    })).toEqual(true);
  });
});