"use strict";

var _utils = require("../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _regex = require("../scripts/components/regex");

var _regex2 = _interopRequireDefault(_regex);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Tests for RegExp utilities.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
describe("Regex utilities", function () {
  it("must prepare strings for RegExp", function () {

    expect(_regex2.default.escapeCharsForRegex("Hello.World?")).toEqual("Hello\\.World\\?", "dots and question marks must be escaped");
    expect(_regex2.default.escapeCharsForRegex("(Hello World)")).toEqual("\\(Hello\\sWorld\\)", "parentheses must be escaped");
    expect(_regex2.default.escapeCharsForRegex("[Hello World]")).toEqual("\\[Hello\\sWorld\\]", "square brackets must be escaped");
    expect(_regex2.default.escapeCharsForRegex("{Hello World}")).toEqual("\\{Hello\\sWorld\\}", "curly braces must be escaped");
    expect(_regex2.default.escapeCharsForRegex("Hello World*")).toEqual("Hello\\sWorld\\*", "stars must be escaped");
    expect(_regex2.default.escapeCharsForRegex("Hello^ World$")).toEqual("Hello\\^\\sWorld\\$", "^ and $ must be escaped");
  });

  it("must allow to obtain regex from multiple words", function () {

    var a = _regex2.default.getPatternFromStrings(["Hello", "World", "Kitty"]);
    expect(a instanceof RegExp).toEqual(true, "return object must be a regex");
    expect(a.source).toEqual("(Hello|World|Kitty)", "words must be alternatives");
    expect(a.flags).toEqual("gim", "pattern must be global, case insensitive and multiline");
  });
});