"use strict";

var _utils = require("../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _sanitizer = require("../scripts/data/sanitizer");

var _sanitizer2 = _interopRequireDefault(_sanitizer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Tests for Sanitizer class.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
var san = new _sanitizer2.default();

describe("Sanitizer class", function () {
  it("must escape strings", function () {
    var a = san.escape("<script>alert('foo');</script>");
    expect(a).toEqual("&lt;script&gt;alert(&#039;foo&#039;);&lt;/script&gt;");
  });

  it("must escape strings inside objects", function () {
    var o = {
      a: {
        b: "<script>alert('foo');</script>"
      }
    };
    var a = san.sanitize(o);
    expect(_utils2.default.equal(o, {
      a: {
        b: "&lt;script&gt;alert(&#039;foo&#039;);&lt;/script&gt;"
      }
    })).toEqual(true);
  });

  it("must escape strings inside arrays", function () {
    var o = {
      a: {
        b: "<script>alert('foo');</script>"
      },
      c: [{
        x: "<script>alert('foo');</script>"
      }]
    };
    var a = san.sanitize(o);
    expect(_utils2.default.equal(o, {
      a: {
        b: "&lt;script&gt;alert(&#039;foo&#039;);&lt;/script&gt;"
      },
      c: [{
        x: "&lt;script&gt;alert(&#039;foo&#039;);&lt;/script&gt;"
      }]
    })).toEqual(true);
  });
});